@@ -17,6 +17,7 @@ Finally, a check is done to make sure that the final ODS is the same as the init
 from __future__ import print_function, division, unicode_literals
 
 import os
+
 os.environ['OMAS_DEBUG_TOPIC'] = '*'
 
 from omas import *
@@ -39,13 +40,18 @@ filename = 'test.nc'
 save_omas_nc(ods, filename)
 ods = load_omas_nc(filename)
 
+# save/load HDF5
+filename = 'test.h5'
+save_omas_h5(ods, filename)
+ods = load_omas_h5(filename)
+
 # remote save/load S3
 filename = 'test.s3'
 save_omas_s3(ods, filename)
 ods = load_omas_s3(filename)
 
 # save/load IMAS
-omas_rcparams['allow_fake_imas_fallback']=True
+omas_rcparams['allow_fake_imas_fallback'] = True
 paths = save_omas_imas(ods, machine='ITER', pulse=1, new=True)
 ods_end = load_omas_imas(machine='ITER', pulse=1, paths=paths)
 
