@@ -15,6 +15,7 @@ try:
 except ImportError as _excp:
     _pyhdc_import_excp = _excp
 
+
     # replace HDC class by a simple exception throwing class
     class HDC(object):
         """Import error HDC class"""
@@ -51,13 +52,13 @@ def save_omas_hdc(ods):
 
 
 def load_omas_hdc(hdc, consistency_check=True):
-    """Convert OMAS data structure to HDC
+    """Convert HDC data structure to OMAS
 
-    :param ods: input data structure
+    :param hdc: input data structure
 
     :param consistency_check: verify that data is consistent with IMAS schema
 
-    :return: HDC container
+    :return: populated ODS
     """
     # recurrent function - check types
     if not isinstance(hdc, HDC):
@@ -83,7 +84,7 @@ def load_omas_hdc(hdc, consistency_check=True):
             # numeric type
             ods = numpy.asarray(hdc)
             if numpy.isscalar(ods) or ods.size == 1:
-                ods = numpy.asscalar(ods)
+                ods = ods.item()
     if consistency_check is not None:
         ods.consistency_check = consistency_check
     return ods
@@ -91,7 +92,7 @@ def load_omas_hdc(hdc, consistency_check=True):
 
 def through_omas_hdc(ods):
     '''
-    test save and load HDC
+    Test save and load HDC
 
     :param ods: ods
 
