@@ -38,11 +38,11 @@ def imas_open(user, machine, pulse, run, new=False,
     if user is None and machine is None:
         pass
     elif user is None or machine is None:
-        raise (Exception('user={user}, machine={machine}, imas_version={imas_version}\n'
-                         'Either specify all or none of `user`, `machine`, `imas_version`\n'
-                         'If none of them are specified then use `imasdb` command to set '
-                         'MDSPLUS_TREE_BASE_? environmental variables'.format(user=user, machine=machine, pulse=pulse,
-                                                                              run=run, imas_version=imas_version)))
+        raise Exception('user={user}, machine={machine}, imas_version={imas_version}\n'
+                        'Either specify all or none of `user`, `machine`, `imas_version`\n'
+                        'If none of them are specified then use `imasdb` command to set '
+                        'MDSPLUS_TREE_BASE_? environmental variables'.format(user=user, machine=machine, pulse=pulse,
+                                                                             run=run, imas_version=imas_version))
 
     if user is None and machine is None:
         if new:
@@ -54,10 +54,10 @@ def imas_open(user, machine, pulse, run, new=False,
                 ids.open()
             except Exception as _excp:
                 if 'Error opening imas pulse' in str(_excp):
-                    raise (IOError('Error opening imas pulse %d run %d' % (pulse, run)))
+                    raise IOError('Error opening imas pulse %d run %d' % (pulse, run))
         if not ids.isConnected():
-            raise (Exception('Failed to establish connection to IMAS database '
-                             '(pulse:{pulse} run:{run}, DB:{db})'.format(pulse=pulse, run=run, db=os.environ.get('MDSPLUS_TREE_BASE_0', '???')[:-2])))
+            raise Exception('Failed to establish connection to IMAS database '
+                            '(pulse:{pulse} run:{run}, DB:{db})'.format(pulse=pulse, run=run, db=os.environ.get('MDSPLUS_TREE_BASE_0', '???')[:-2]))
 
     else:
         if new:
@@ -69,9 +69,9 @@ def imas_open(user, machine, pulse, run, new=False,
                 ids.open_env(user, machine, imas_version)
             except Exception as _excp:
                 if 'Error opening imas pulse' in str(_excp):
-                    raise (IOError('Error opening imas pulse (user:%s machine:%s pulse:%s run:%s, imas_version:%s)' % (user, machine, pulse, run, imas_version)))
+                    raise IOError('Error opening imas pulse (user:%s machine:%s pulse:%s run:%s, imas_version:%s)' % (user, machine, pulse, run, imas_version))
         if not ids.isConnected():
-            raise (Exception('Failed to establish connection to IMAS database (user:%s machine:%s pulse:%s run:%s, imas_version:%s)' % (user, machine, pulse, run, imas_version)))
+            raise Exception('Failed to establish connection to IMAS database (user:%s machine:%s pulse:%s run:%s, imas_version:%s)' % (user, machine, pulse, run, imas_version))
     return ids
 
 
@@ -110,11 +110,6 @@ def imas_set(ids, path, value, skip_missing_nodes=False, allocate=False):
     if ds in add_datastructures.keys():
         return
 
-    # for ITM we have to append Array to the name of the data structure
-    DS = ds
-    if 'imas' == 'itm':
-        ds = ds + 'Array'
-
     # identify data dictionary to use, from this point on `m` points to the IDS
     debug_path = ''
     if hasattr(ids, ds):
@@ -125,11 +120,11 @@ def imas_set(ids, path, value, skip_missing_nodes=False, allocate=False):
             m.time[0] = -1.0
     elif skip_missing_nodes is not False:
         if skip_missing_nodes is None:
-            printe('WARNING: %s is not part of IMAS structure' % l2i([ds] + path))
+            printe('WARNING: %s is not part of IMAS' % l2i([ds] + path))
         return
     else:
         printd(debug_path, topic='imas_code')
-        raise (AttributeError('%s is not part of IMAS structure' % l2i([ds] + path)))
+        raise AttributeError('%s is not part of IMAS' % l2i([ds] + path))
 
     # traverse IMAS structure until reaching the leaf
     out = m
@@ -142,18 +137,18 @@ def imas_set(ids, path, value, skip_missing_nodes=False, allocate=False):
                     out = getattr(out, p)
             elif skip_missing_nodes is not False:
                 if skip_missing_nodes is None:
-                    printe('WARNING: %s is not part of IMAS structure' % location)
+                    printe('WARNING: %s is not part of IMAS' % location)
                 return
             else:
                 printd(debug_path, topic='imas_code')
-                raise (AttributeError('%s is not part of IMAS structure' % location))
+                raise AttributeError('%s is not part of IMAS' % location)
         else:
             try:
                 out = out[p]
                 debug_path += '[%d]' % p
-            except (AttributeError, IndexError):  # AttributeError is for ITM
+            except IndexError:
                 if not allocate:
-                    raise (IndexError('%s structure array exceed allocation' % location))
+                    raise IndexError('%s structure array exceed allocation' % location)
                 printd(debug_path + ".resize(%d)" % (p + 1), topic='imas_code')
                 out.resize(p + 1)
                 debug_path += '[%d]' % p
@@ -161,7 +156,7 @@ def imas_set(ids, path, value, skip_missing_nodes=False, allocate=False):
 
     # if we are allocating data, simply stop here
     if allocate:
-        return [DS] + path
+        return [ds] + path
 
     # assign data to leaf node
     printd('setting  : %s' % location, topic='imas')
@@ -172,7 +167,30 @@ def imas_set(ids, path, value, skip_missing_nodes=False, allocate=False):
         printd(debug_path + '.%s=%s' % (path[-1], repr(value).replace('\\n', '\n')), topic='imas_code')
 
     # return path
-    return [DS] + path
+    return [ds] + path
+
+
+def imas_empty(value):
+    '''
+    Check if value is an IMAS empty
+        * array with no size
+        * float of value -9E40
+        * integer of value -999999999
+        * empty string
+
+    :param value: value to check
+
+    :return: None if value is an IMAS empty
+    '''
+    if isinstance(value, numpy.ndarray) and not value.size:
+        value = None
+    # missing floats and integers
+    elif (isinstance(value, float) and value == -9E40) or (isinstance(value, int) and value == -999999999):
+        value = None
+    # empty strings
+    elif isinstance(value, basestring) and not len(value):
+        value = None
+    return value
 
 
 def imas_get(ids, path, skip_missing_nodes=False):
@@ -194,21 +212,17 @@ def imas_get(ids, path, skip_missing_nodes=False):
     ds = path[0]
     path = path[1:]
 
-    # for ITM we have to append Array to the name of the data structure
-    if 'imas' == 'itm':
-        ds = ds + 'Array'
-
     debug_path = ''
     if hasattr(ids, ds):
         debug_path += 'ids.%s' % ds
         m = getattr(ids, ds)
     elif skip_missing_nodes is not False:
         if skip_missing_nodes is None:
-            printe('WARNING: %s is not part of IMAS structure' % l2i([ds] + path))
+            printe('WARNING: %s is not part of IMAS' % l2i([ds] + path))
         return None
     else:
         printd(debug_path, topic='imas_code')
-        raise (AttributeError('%s is not part of IMAS structure' % l2i([ds] + path)))
+        raise AttributeError('%s is not part of IMAS' % l2i([ds] + path))
 
     # traverse the IDS to get the data
     out = m
@@ -219,29 +233,18 @@ def imas_get(ids, path, skip_missing_nodes=False):
                 out = getattr(out, p)
             elif skip_missing_nodes is not False:
                 if skip_missing_nodes is None:
-                    printe('WARNING: %s is not part of IMAS structure' % l2i([ds] + path[:kp + 1]))
+                    printe('WARNING: %s is not part of IMAS' % l2i([ds] + path[:kp + 1]))
                     printe(out.__dict__.keys())
                 return None
             else:
                 printd(debug_path, topic='imas_code')
-                raise (AttributeError('%s is not part of IMAS structure' % l2i([ds] + path[:kp + 1])))
+                raise AttributeError('%s is not part of IMAS' % l2i([ds] + path[:kp + 1]))
         else:
             debug_path += '[%s]' % p
             out = out[p]
 
     # handle missing data
-    data = out
-    if len(path) == 2 and path[-1] == 'time' and data[0] < 0:
-        data = None
-    # skip empty arrays
-    elif isinstance(data, numpy.ndarray) and not data.size:
-        data = None
-    # skip missing floats and integers
-    elif (isinstance(data, float) and data == -9E40) or (isinstance(data, int) and data == -999999999):
-        data = None
-    # skip empty strings
-    elif isinstance(data, unicode) and not len(data):
-        data = None
+    data = imas_empty(out)
 
     printd(debug_path, topic='imas_code')
     return data
@@ -301,9 +304,7 @@ def save_omas_imas(ods, user=None, machine=None, pulse=None, run=None, new=False
         printd('Saving to IMAS (pulse:%d run:%d, DB:%s)' % (pulse, run, os.environ.get('MDSPLUS_TREE_BASE_0', '???')[:-2]), topic='imas')
 
     # ensure requirements for writing data to IMAS are satisfied
-    if 'imas' != 'itm':
-        for ds in ods.keys():
-            ods[ds].satisfy_imas_requirements()
+    ods.satisfy_imas_requirements()
 
     # get the list of paths from ODS
     paths = set_paths = ods.paths()
@@ -313,7 +314,7 @@ def save_omas_imas(ods, user=None, machine=None, pulse=None, run=None, new=False
         ids = imas_open(user=user, machine=machine, pulse=pulse, run=run, new=new, imas_version=imas_version)
 
     except IOError as _excp:
-        raise (IOError(str(_excp) + '\nIf this is a new pulse/run then set `new=True`'))
+        raise IOError(str(_excp) + '\nIf this is a new pulse/run then set `new=True`')
 
     except ImportError:
         # fallback on saving IMAS as NC file if IMAS is not installed
@@ -350,8 +351,6 @@ def save_omas_imas(ods, user=None, machine=None, pulse=None, run=None, new=False
             for ds in ods.keys():
                 if ds in add_datastructures.keys():
                     continue
-                if 'imas' == 'itm':
-                    ds = ds + 'Array'
                 printd("ids.%s.put(0)" % ds, topic='imas_code')
                 getattr(ids, ds).put(0)
 
@@ -363,8 +362,9 @@ def save_omas_imas(ods, user=None, machine=None, pulse=None, run=None, new=False
     return set_paths
 
 
-def load_omas_imas(user=os.environ['USER'], machine=None, pulse=None, run=0, paths=None,
-                   imas_version=os.environ.get('IMAS_VERSION', omas_rcparams['default_imas_version']), verbose=True):
+def load_omas_imas(user=os.environ.get('USER', 'dummy_user'), machine=None, pulse=None, run=0, paths=None,
+                   imas_version=os.environ.get('IMAS_VERSION', omas_rcparams['default_imas_version']),
+                   skip_uncertainties=False, skip_ggd=True, verbose=True):
     """
     Load OMAS data from IMAS
 
@@ -383,13 +383,17 @@ def load_omas_imas(user=os.environ['USER'], machine=None, pulse=None, run=0, pat
 
     :param imas_version: IMAS version
 
+    :param skip_uncertainties: do not load uncertain data
+
+    :param skip_ggd: do not load ggd structure
+
     :param verbose: print loading progress
 
     :return: OMAS data set
     """
 
     if pulse is None or run is None:
-        raise (Exception('`pulse` and `run` must be specified'))
+        raise Exception('`pulse` and `run` must be specified')
 
     printd('Loading from IMAS (user:%s machine:%s pulse:%d run:%d, imas_version:%s)' % (user, machine, pulse, run, imas_version), topic='imas')
 
@@ -409,16 +413,13 @@ def load_omas_imas(user=os.environ['USER'], machine=None, pulse=None, run=0, pat
         try:
             # if paths is None then figure out what IDS are available and get ready to retrieve everything
             if paths is None:
-                paths = [[structure] for structure in list_structures(imas_version=imas_version)]
-                joined_paths = None # joined_paths==None means take everything, no need to filter out what is available based on what was requested
+                requested_paths = [[structure] for structure in list_structures(imas_version=imas_version)]
             else:
-                joined_paths = map(l2i, paths)
+                requested_paths = map(p2l, paths)
 
             # fetch relevant IDSs and find available signals
             fetch_paths = []
-            for path in paths:
-                ds = path[0]
-                path = path[1:]
+            for ds in numpy.unique([p[0] for p in requested_paths]):
                 if ds in add_datastructures.keys():
                     continue
                 if not hasattr(ids, ds):
@@ -433,40 +434,26 @@ def load_omas_imas(user=os.environ['USER'], machine=None, pulse=None, run=0, pat
                 if len(getattr(ids, ds).time):
                     if verbose:
                         print('* ', ds)
-                    available_paths = filled_paths_in_ids(ids, load_structure(ds, imas_version=imas_version)[1], [], [])
-                    if joined_paths is None:
-                        # if joined_paths is None, this means the user requested everything
-                        fetch_paths = available_paths
-                    else:
-                        # intersect between what was requested and what is available
-                        joined_available_paths = map(l2i, available_paths)
-                        for jpath, path in zip(joined_paths, paths):
-                            if path[0] != ds:
-                                continue
-                            jpath = jpath.replace('.', '\.')
-                            jpath = '^' + jpath.replace('.:', '.[0-9]+') + '.*'
-                            for japath, apath in zip(joined_available_paths, available_paths):
-                                if re.match(jpath, japath):
-                                    fetch_paths.append(apath)
+                    fetch_paths += filled_paths_in_ids(ids, load_structure(ds, imas_version=imas_version)[1], [], [], requested_paths, skip_ggd=skip_ggd)
                 else:
                     if verbose:
                         print('- ', ds)
             joined_fetch_paths = map(l2i, fetch_paths)
 
             # build omas data structure
-            ods = ODS(imas_version=imas_version)
+            ods = ODS(imas_version=imas_version, consistency_check=False)
             for k, path in enumerate(fetch_paths):
-                if path[-1].endswith('_error_upper') or path[-1].endswith('_error_lower'):
+                if path[-1].endswith('_error_upper') or path[-1].endswith('_error_lower') or path[-1].endswith('_error_index'):
                     continue
-                if verbose:
-                    print('Loading data: {0:3.3f}%'.format(100 * float(k) / len(fetch_paths)))
-                # get data from ids
+                if verbose and (k % 100 == 0 or k == len(fetch_paths) - 1):
+                    print('Loading {0:3.3f}%'.format(100 * float(k) / (len(fetch_paths) - 1)))
+                # get data from IDS
                 data = imas_get(ids, path, None)
                 # continue for empty data
                 if data is None:
                     continue
                 # add uncertainty
-                if l2i(path[:-1] + [path[-1] + '_error_upper']) in joined_fetch_paths:
+                if not skip_uncertainties and l2i(path[:-1] + [path[-1] + '_error_upper']) in joined_fetch_paths:
                     stdata = imas_get(ids, path[:-1] + [path[-1] + '_error_upper'], None)
                     if stdata is not None:
                         try:
@@ -474,129 +461,131 @@ def load_omas_imas(user=os.environ['USER'], machine=None, pulse=None, run=0, pat
                         except uncertainties.core.NegativeStdDev as _excp:
                             printe('Error loading uncertainty for %s: %s' % (l2i(path), repr(_excp)))
                 # assign data to ODS
-                h = ods
-                for step in path[:-1]:
-                    h = h[step]
-                h[path[-1]] = data
+                ods[path] = data
 
         finally:
             # close connection to IMAS database
             printd("ids.close()", topic='imas_code')
             ids.close()
 
-    ods['dataset_description.data_entry.user'] = unicode(user)
-    ods['dataset_description.data_entry.machine'] = unicode(machine)
-    ods['dataset_description.data_entry.pulse'] = int(pulse)
-    ods['dataset_description.data_entry.run'] = int(run)
-    ods['dataset_description.imas_version'] = unicode(imas_version)
+    if paths is None:
+        ods.setdefault('dataset_description.data_entry.user', unicode(user))
+        ods.setdefault('dataset_description.data_entry.machine', unicode(machine))
+        ods.setdefault('dataset_description.data_entry.pulse', int(pulse))
+        ods.setdefault('dataset_description.data_entry.run', int(run))
+        ods.setdefault('dataset_description.imas_version', unicode(imas_version))
+
+    try:
+        ods.consistency_check = True
+    except LookupError as _excp:
+        printe(repr(excp))
 
     return ods
 
 
-if 'imas' != 'itm':
-    def browse_imas(user=os.environ['USER'], pretty=True, quiet=False,
-                    user_imasdbdir=os.sep.join([os.environ['HOME'], 'public', 'imasdb'])):
-        '''
-        Browse available IMAS data (machine/pulse/run) for given user
+def browse_imas(user=os.environ.get('USER', 'dummy_user'), pretty=True, quiet=False,
+                user_imasdbdir=os.sep.join([os.environ['HOME'], 'public', 'imasdb'])):
+    '''
+    Browse available IMAS data (machine/pulse/run) for given user
 
-        :param user: user (of list of users) to browse. Browses all users if None.
+    :param user: user (of list of users) to browse. Browses all users if None.
 
-        :param pretty: express size in MB and time in human readeable format
+    :param pretty: express size in MB and time in human readeable format
 
-        :param quiet: print database to screen
+    :param quiet: print database to screen
 
-        :param user_imasdbdir: directory where imasdb is located for current user (typically $HOME/public/imasdb/)
+    :param user_imasdbdir: directory where imasdb is located for current user (typically $HOME/public/imasdb/)
 
-        :return: hierarchical dictionary with database of available IMAS data (machine/pulse/run) for given user
-        '''
-        # if no users are specified, find all users
-        if user is None:
-            user = glob.glob(user_imasdbdir.replace('/%s/' % os.environ['USER'], '/*/'))
-            user = map(lambda x: x.split(os.sep)[-3], user)
-        elif isinstance(user, basestring):
-            user = [user]
+    :return: hierarchical dictionary with database of available IMAS data (machine/pulse/run) for given user
+    '''
+    # if no users are specified, find all users
+    if user is None:
+        user = glob.glob(user_imasdbdir.replace('/%s/' % os.environ['USER'], '/*/'))
+        user = map(lambda x: x.split(os.sep)[-3], user)
+    elif isinstance(user, basestring):
+        user = [user]
 
-        # build database for each user
-        imasdb = {}
-        for username in user:
-            imasdb[username] = {}
-            imasdbdir = user_imasdbdir.replace('/%s/' % os.environ['USER'], '/%s/' % username).strip()
+    # build database for each user
+    imasdb = {}
+    for username in user:
+        imasdb[username] = {}
+        imasdbdir = user_imasdbdir.replace('/%s/' % os.environ['USER'], '/%s/' % username).strip()
 
-            # find MDS+ datafiles
-            files = list(recursive_glob('*datafile', imasdbdir))
+        # find MDS+ datafiles
+        files = list(recursive_glob('*datafile', imasdbdir))
 
-            # extract machine/pulse/run from filename of MDS+ datafiles
-            for file in files:
-                tmp = file.split(os.sep)
-                if not re.match('ids_[0-9]{5,}.datafile', tmp[-1]):
-                    continue
-                pulse_run = tmp[-1].split('.')[0].split('_')[1]
-                pulse = int(pulse_run[:-4])
-                run = int(pulse_run[-4:])
-                machine = tmp[-4]
+        # extract machine/pulse/run from filename of MDS+ datafiles
+        for file in files:
+            tmp = file.split(os.sep)
+            if not re.match('ids_[0-9]{5,}.datafile', tmp[-1]):
+                continue
+            pulse_run = tmp[-1].split('.')[0].split('_')[1]
+            pulse = int(pulse_run[:-4])
+            run = int(pulse_run[-4:])
+            machine = tmp[-4]
 
-                # size and data
-                st = os.stat(file)
-                size = st.st_size
-                date = st.st_mtime
-                if pretty:
-                    import time
-                    size = '%d Mb' % (int(size / 1024 / 1024))
-                    date = time.strftime('%d/%m/%y - %H:%M', time.localtime(date))
+            # size and data
+            st = os.stat(file)
+            size = st.st_size
+            date = st.st_mtime
+            if pretty:
+                import time
+                size = '%d Mb' % (int(size / 1024 / 1024))
+                date = time.strftime('%d/%m/%y - %H:%M', time.localtime(date))
 
-                # build database
-                if machine not in imasdb[username]:
-                    imasdb[username][machine] = {}
-                imasdb[username][machine][pulse, run] = {'size': size, 'date': date}
+            # build database
+            if machine not in imasdb[username]:
+                imasdb[username][machine] = {}
+            imasdb[username][machine][pulse, run] = {'size': size, 'date': date}
 
-        # print if not quiet
-        if not quiet:
-            pprint(imasdb)
+    # print if not quiet
+    if not quiet:
+        pprint(imasdb)
 
-        # return database
-        return imasdb
+    # return database
+    return imasdb
 
 
-    def load_omas_iter_scenario(pulse, run=0, paths=None,
-                                imas_version=os.environ.get('IMAS_VERSION', omas_rcparams['default_imas_version']),
-                                verbose=True):
-        """
-        Load OMAS data set from ITER IMAS scenario database
-    
-        :param pulse: IMAS pulse
-    
-        :param run: IMAS run
-    
-        :param paths: list of paths to load from IMAS
-    
-        :param imas_version: IMAS version
-    
-        :return: OMAS data set
-    
-        :param verbose: print loading progress
-        """
-        # set MDSPLUS_TREE_BASE_? environment variables as per
-        # imasdb /work/imas/shared/iterdb/3 ; env | grep MDSPLUS_TREE_BASE
-        try:
-            bkp_imas_environment = {}
-            for k in range(10):
-                if 'MDSPLUS_TREE_BASE_%d' % k in os.environ:
-                    bkp_imas_environment['MDSPLUS_TREE_BASE_%d' % k] = os.environ['MDSPLUS_TREE_BASE_%d' % k]
-                os.environ['MDSPLUS_TREE_BASE_%d' % k] = '/work/imas/shared/iterdb/3/%d' % k
+def load_omas_iter_scenario(pulse, run=0, paths=None,
+                            imas_version=os.environ.get('IMAS_VERSION', omas_rcparams['default_imas_version']),
+                            verbose=True):
+    """
+    Load OMAS data set from ITER IMAS scenario database
 
-            # load data from imas
-            ods = load_omas_imas(user=None, machine=None, pulse=pulse, run=run, paths=paths, imas_version=imas_version, verbose=verbose)
+    :param pulse: IMAS pulse
 
-        finally:
-            # restore existing IMAS environment
-            for k in range(10):
-                del os.environ['MDSPLUS_TREE_BASE_%d' % k]
-                os.environ.update(bkp_imas_environment)
+    :param run: IMAS run
 
-        return ods
+    :param paths: list of paths to load from IMAS
+
+    :param imas_version: IMAS version
+
+    :param verbose: print loading progress
+
+    :return: OMAS data set
+    """
+    # set MDSPLUS_TREE_BASE_? environment variables as per
+    # imasdb /work/imas/shared/iterdb/3 ; env | grep MDSPLUS_TREE_BASE
+    try:
+        bkp_imas_environment = {}
+        for k in range(10):
+            if 'MDSPLUS_TREE_BASE_%d' % k in os.environ:
+                bkp_imas_environment['MDSPLUS_TREE_BASE_%d' % k] = os.environ['MDSPLUS_TREE_BASE_%d' % k]
+            os.environ['MDSPLUS_TREE_BASE_%d' % k] = '/work/imas/shared/iterdb/3/%d' % k
+
+        # load data from imas
+        ods = load_omas_imas(user=None, machine=None, pulse=pulse, run=run, paths=paths, imas_version=imas_version, verbose=verbose)
+
+    finally:
+        # restore existing IMAS environment
+        for k in range(10):
+            del os.environ['MDSPLUS_TREE_BASE_%d' % k]
+            os.environ.update(bkp_imas_environment)
+
+    return ods
 
 
-def filled_paths_in_ids(ids, ds, path=None, paths=None, assume_uniform_array_structures=False):
+def filled_paths_in_ids(ids, ds, path=None, paths=None, requested_paths=None, assume_uniform_array_structures=False, skip_ggd=True):
     """
     Taverse an IDS and list leaf paths (with proper sizing for arrays of structures)
 
@@ -604,8 +593,12 @@ def filled_paths_in_ids(ids, ds, path=None, paths=None, assume_uniform_array_str
 
     :param ds: hierarchical data schema as returned for example by load_structure('equilibrium')[1]
 
+    :param requested_paths: list of paths that are requested
+
     :param assume_uniform_array_structures: assume that the first structure in an array of structures has data in the same nodes locations of the later structures in the array
 
+    :param skip_ggd: do not traverse ggd structures
+
     :return: returns list of paths in an IDS that are filled
     """
     if path is None:
@@ -614,31 +607,54 @@ def filled_paths_in_ids(ids, ds, path=None, paths=None, assume_uniform_array_str
     if paths is None:
         paths = []
 
+    if requested_paths is None:
+        requested_paths = []
+
     # leaf
     if not len(ds):
-        paths.append(path)
-        #print(paths[-1])
+        # append path if it has data
+        if imas_empty(ids) is not None:
+            paths.append(path)
         return paths
 
     # keys
-    keys = ds.keys()
+    keys = list(ds.keys())
     if keys[0] == ':':
         keys = range(len(ids))
         if len(keys) and assume_uniform_array_structures:
             keys = [0]
 
+    # kid must be part of this list
+    if len(requested_paths):
+        request_check = [p[0] for p in requested_paths]
+
     # traverse
     for kid in keys:
+
+        # skip ggd structures
+        if skip_ggd and kid in ['ggd', 'grids_ggd']:
+            continue
+
         propagate_path = copy.copy(path)
         propagate_path.append(kid)
+
+        # generate requested_paths one level deeper
+        propagate_requested_paths = requested_paths
+        if len(requested_paths):
+            if kid in request_check or (isinstance(kid, int) and ':' in request_check):
+                propagate_requested_paths = [p[1:] for p in requested_paths if len(p) > 1 and (kid == p[0] or p[0] == ':')]
+            else:
+                continue
+
+        # recursive call
         if isinstance(kid, basestring):
-            subtree_paths = filled_paths_in_ids(getattr(ids, kid), ds[kid], propagate_path, [], assume_uniform_array_structures)
+            subtree_paths = filled_paths_in_ids(getattr(ids, kid), ds[kid], propagate_path, [], propagate_requested_paths, assume_uniform_array_structures, skip_ggd=skip_ggd)
         else:
-            subtree_paths = filled_paths_in_ids(ids[kid], ds[':'], propagate_path, [], assume_uniform_array_structures)
+            subtree_paths = filled_paths_in_ids(ids[kid], ds[':'], propagate_path, [], propagate_requested_paths, assume_uniform_array_structures, skip_ggd=skip_ggd)
         paths += subtree_paths
 
         # assume_uniform_array_structures
-        if keys[0]==0 and assume_uniform_array_structures:
+        if assume_uniform_array_structures and keys[0] == 0:
             zero_paths = subtree_paths
             for key in range(1, len(ids)):
                 subtree_paths = copy.deepcopy(zero_paths)
@@ -651,7 +667,7 @@ def filled_paths_in_ids(ids, ds, path=None, paths=None, assume_uniform_array_str
 
 def through_omas_imas(ods):
     """
-    test save and load OMAS IMAS
+    Test save and load OMAS IMAS
 
     :param ods: ods
 
