@@ -34,7 +34,7 @@ def save_omas_json(ods, filename, objects_encode=None, **kw):
     kw.setdefault('separators', (',', ': '))
     kw.setdefault('sort_keys', True)
 
-    json_string = json.dumps(ods, default=lambda x:json_dumper(x,objects_encode), **kw)
+    json_string = json.dumps(ods, default=lambda x: json_dumper(x, objects_encode), **kw)
 
     if isinstance(filename, basestring):
         with open(filename, 'w') as f:
@@ -44,7 +44,7 @@ def save_omas_json(ods, filename, objects_encode=None, **kw):
         f.write(json_string)
 
 
-def load_omas_json(filename, consistency_check=True, **kw):
+def load_omas_json(filename, consistency_check=True, imas_version=omas_rcparams['default_imas_version'], **kw):
     """
     Load an OMAS data set from Json
 
@@ -52,6 +52,8 @@ def load_omas_json(filename, consistency_check=True, **kw):
 
     :param consistency_check: verify that data is consistent with IMAS schema
 
+    :param imas_version: imas version to use for consistency check
+
     :param kw: arguments passed to the json.loads mehtod
 
     :return: OMAS data set
@@ -60,7 +62,7 @@ def load_omas_json(filename, consistency_check=True, **kw):
     printd('Loading OMAS data to Json: %s' % filename, topic='json')
 
     def cls():
-        tmp = ODS()
+        tmp = ODS(imas_version=imas_version)
         tmp.consistency_check = False
         return tmp
 
@@ -78,7 +80,7 @@ def load_omas_json(filename, consistency_check=True, **kw):
 
 def through_omas_json(ods):
     """
-    test save and load OMAS Json
+    Test save and load OMAS Json
 
     :param ods: ods
 
