@@ -14,7 +14,7 @@ from .omas_core import ODS
 # --------------------------------------------
 def save_omas_nc(ods, filename, **kw):
     """
-    Save an OMAS data set to on Amazon S3 server
+    Save an OMAS data set to NetCDF file
 
     :param ods: OMAS data set
 
@@ -30,11 +30,11 @@ def save_omas_nc(ods, filename, **kw):
         for item in odsf:
             dims = []
             data = numpy.asarray(odsf[item])
-            std  = None
-            tmp=is_uncertain(odsf[item])
+            std = None
+            tmp = is_uncertain(odsf[item])
             if numpy.any(numpy.atleast_1d(tmp)):
-                std=std_devs(data)
-                data=nominal_values(data)
+                std = std_devs(data)
+                data = nominal_values(data)
             for k in range(len(numpy.asarray(odsf[item]).shape)):
                 dims.append('dim_%d' % (data.shape[k]))
                 if dims[-1] not in dataset.dimensions:
@@ -43,12 +43,12 @@ def save_omas_nc(ods, filename, **kw):
                 dataset.createVariable(item, data.dtype, dims)[:] = data
             else:
                 dataset.createVariable(item, data.dtype, dims)[:] = data
-                dataset.createVariable(item+'_error_upper', data.dtype, dims)[:] = std
+                dataset.createVariable(item + '_error_upper', data.dtype, dims)[:] = std
 
 
 def load_omas_nc(filename, consistency_check=True, ):
     """
-    Load an OMAS data set from Amazon S3 server
+    Load an OMAS data set from NetCDF file
 
     :param filename: filename to load from
 
@@ -74,12 +74,12 @@ def load_omas_nc(filename, consistency_check=True, ):
             else:
                 # uncertain scalars
                 if item + '_error_upper' in dataset.variables.keys():
-                    ods[item] = ufloat(numpy.asscalar(dataset.variables[item][0]),
-                                       numpy.asscalar(dataset.variables[item + '_error_upper'][0]))
+                    ods[item] = ufloat(dataset.variables[item][0].item(),
+                                       dataset.variables[item + '_error_upper'][0].item())
                 else:
                     try:
                         # scalars
-                        ods[item] = numpy.asscalar(dataset.variables[item][0])
+                        ods[item] = dataset.variables[item][0].item()
                     except AttributeError:
                         # strings
                         ods[item] = dataset.variables[item][0]
@@ -89,15 +89,15 @@ def load_omas_nc(filename, consistency_check=True, ):
 
 def through_omas_nc(ods):
     """
-    test save and load NetCDF
+    Test save and load NetCDF
 
     :param ods: ods
 
     :return: ods
     """
-    if not os.path.exists(tempfile.gettempdir()+'/OMAS_TESTS/'):
-        os.makedirs(tempfile.gettempdir()+'/OMAS_TESTS/')
-    filename = tempfile.gettempdir()+'/OMAS_TESTS/test.nc'
+    if not os.path.exists(tempfile.gettempdir() + '/OMAS_TESTS/'):
+        os.makedirs(tempfile.gettempdir() + '/OMAS_TESTS/')
+    filename = tempfile.gettempdir() + '/OMAS_TESTS/test.nc'
     save_omas_nc(ods, filename)
     ods1 = load_omas_nc(filename)
     return ods1
