@@ -10,6 +10,7 @@ from .omas_core import ODS
 __all__ = []
 __ods__ = []
 
+
 def add_to__ODS__(f):
     '''
     anything wrapped here will be available as a ODS method with name 'physics_'+f.__name__
@@ -18,14 +19,17 @@ def add_to__ODS__(f):
     __all__.append(f.__name__)
     return f
 
+
 def add_to__ALL__(f):
     __all__.append(f.__name__)
     return f
 
+
 # constants class that mimics scipy.constants
 class constants(object):
     e = 1.6021766208e-19
 
+
 @add_to__ODS__
 def core_profiles_consistent(ods, update=True, use_electrons_density=False):
     '''
@@ -49,6 +53,7 @@ def core_profiles_consistent(ods, update=True, use_electrons_density=False):
     core_profiles_zeff(ods, update=True, use_electrons_density=use_electrons_density)
     return ods
 
+
 @add_to__ODS__
 def core_profiles_pressures(ods, update=True):
     '''
@@ -87,12 +92,12 @@ def core_profiles_pressures(ods, update=True):
         if not update:
             prof1d_p['grid']['rho_tor_norm'] = prof1d['grid']['rho_tor_norm']
 
-        __zeros__ = 0.*prof1d['grid']['rho_tor_norm']
+        __zeros__ = 0. * prof1d['grid']['rho_tor_norm']
 
-        prof1d_p['pressure_thermal']       = copy.deepcopy(__zeros__)
-        prof1d_p['pressure_ion_total']     = copy.deepcopy(__zeros__)
+        prof1d_p['pressure_thermal'] = copy.deepcopy(__zeros__)
+        prof1d_p['pressure_ion_total'] = copy.deepcopy(__zeros__)
         prof1d_p['pressure_perpendicular'] = copy.deepcopy(__zeros__)
-        prof1d_p['pressure_parallel']      = copy.deepcopy(__zeros__)
+        prof1d_p['pressure_parallel'] = copy.deepcopy(__zeros__)
 
         # electrons
         prof1d_p['electrons']['pressure'] = copy.deepcopy(__zeros__)
@@ -105,16 +110,16 @@ def core_profiles_pressures(ods, update=True):
 
         if __p__ is not None:
             prof1d_p['electrons']['pressure_thermal'] = __p__
-            prof1d_p['electrons']['pressure']  += __p__
-            prof1d_p['pressure_thermal']       += __p__
+            prof1d_p['electrons']['pressure'] += __p__
+            prof1d_p['pressure_thermal'] += __p__
             prof1d_p['pressure_perpendicular'] += __p__ / 3.
-            prof1d_p['pressure_parallel']      += __p__ / 3.
+            prof1d_p['pressure_parallel'] += __p__ / 3.
 
         if 'pressure_fast_perpendicular' in prof1d['electrons']:
             __p__ = nominal_values(prof1d['electrons']['pressure_fast_perpendicular'])
             if not update:
                 prof1d_p['electrons']['pressure_fast_perpendicular'] = __p__
-            prof1d_p['electrons']['pressure']  += 2. * __p__
+            prof1d_p['electrons']['pressure'] += 2. * __p__
             prof1d_p['pressure_perpendicular'] += __p__
 
         if 'pressure_fast_parallel' in prof1d['electrons']:
@@ -122,9 +127,9 @@ def core_profiles_pressures(ods, update=True):
             if not update:
                 prof1d_p['electrons']['pressure_fast_parallel'] = __p__
             prof1d_p['electrons']['pressure'] += __p__
-            prof1d_p['pressure_parallel']     += __p__
+            prof1d_p['pressure_parallel'] += __p__
 
-        #ions
+        # ions
         for k in range(len(prof1d['ion'])):
 
             prof1d_p['ion'][k]['pressure'] = copy.deepcopy(__zeros__)
@@ -137,27 +142,27 @@ def core_profiles_pressures(ods, update=True):
 
             if __p__ is not None:
                 prof1d_p['ion'][k]['pressure_thermal'] = __p__
-                prof1d_p['ion'][k]['pressure']     += __p__
-                prof1d_p['pressure_thermal']       += __p__
+                prof1d_p['ion'][k]['pressure'] += __p__
+                prof1d_p['pressure_thermal'] += __p__
                 prof1d_p['pressure_perpendicular'] += __p__ / 3.
-                prof1d_p['pressure_parallel']      += __p__ / 3.
-                prof1d_p['pressure_ion_total']     += __p__
+                prof1d_p['pressure_parallel'] += __p__ / 3.
+                prof1d_p['pressure_ion_total'] += __p__
 
             if 'pressure_fast_perpendicular' in prof1d['ion'][k]:
                 __p__ = nominal_values(prof1d['ion'][k]['pressure_fast_perpendicular'])
                 if not update:
                     prof1d_p['ion'][k]['pressure_fast_perpendicular'] = __p__
-                prof1d_p['ion'][k]['pressure']     += 2. * __p__
+                prof1d_p['ion'][k]['pressure'] += 2. * __p__
                 prof1d_p['pressure_perpendicular'] += __p__
 
             if 'pressure_fast_parallel' in prof1d['ion'][k]:
                 __p__ = nominal_values(prof1d['ion'][k]['pressure_fast_parallel'])
                 if not update:
                     prof1d_p['ion'][k]['pressure_fast_parallel'] = __p__
-                prof1d_p['ion'][k]['pressure'] +=  __p__
-                prof1d_p['pressure_parallel']  += __p__
+                prof1d_p['ion'][k]['pressure'] += __p__
+                prof1d_p['pressure_parallel'] += __p__
 
-        #extra pressure information that is not within IMAS structure is set only if consistency_check is not True
+        # extra pressure information that is not within IMAS structure is set only if consistency_check is not True
         if ods_p.consistency_check is not True:
             prof1d_p['pressure'] = prof1d_p['pressure_perpendicular'] * 2 + prof1d_p['pressure_parallel']
             prof1d_p['pressure_electron_total'] = prof1d_p['pressure_thermal'] - prof1d_p['pressure_ion_total']
@@ -169,7 +174,7 @@ def core_profiles_pressures(ods, update=True):
 @add_to__ODS__
 def core_profiles_densities(ods, update=True):
     '''
-    calculates density from density_thermal and density_fast
+    Density, density_thermal, and density_fast for electrons and ions are filled and are self-consistent
 
     :param ods: input ods
 
@@ -183,6 +188,29 @@ def core_profiles_densities(ods, update=True):
         from omas import ODS
         ods_n = ODS().copy_attrs_from(ods)
 
+    def consistent_density(loc):
+        if 'density' in loc:
+
+            # if there is no thermal nor fast, assume it is thermal
+            if 'density_thermal' not in loc and 'density_fast' not in loc:
+                loc['density_thermal'] = loc['density']
+
+            # if there is no thermal calculate it
+            elif 'density_thermal' not in loc and 'density_fast' in loc:
+                loc['density_thermal'] = loc['density'] - loc['density_fast']
+
+            # if there is no fast calculate it
+            elif 'density_thermal' in loc and 'density_fast' not in loc:
+                loc['density_fast'] = loc['density'] - loc['density_thermal']
+
+        # enforce self-consistency
+        loc['density'] = copy.deepcopy(__zeros__)
+        for density in ['density_thermal', 'density_fast']:
+            if density in loc:
+                loc['density'] += loc[density]
+            else:
+                loc[density] = copy.deepcopy(__zeros__)
+
     for time_index in ods['core_profiles']['profiles_1d']:
         prof1d = ods['core_profiles']['profiles_1d'][time_index]
         prof1d_n = ods_n['core_profiles']['profiles_1d'][time_index]
@@ -190,22 +218,18 @@ def core_profiles_densities(ods, update=True):
         if not update:
             prof1d_n['grid']['rho_tor_norm'] = prof1d['grid']['rho_tor_norm']
 
-        __zeros__ = 0.*prof1d['grid']['rho_tor_norm']
+        __zeros__ = 0. * prof1d['grid']['rho_tor_norm']
 
         # electrons
-        prof1d_n['electrons']['density'] = copy.deepcopy(__zeros__)
-        for density in ['density_thermal','density_fast']:
-            if density in prof1d['electrons']:
-                 prof1d_n['electrons']['density'] += prof1d['electrons'][density]
+        consistent_density(prof1d_n['electrons'])
 
         # ions
         for k in range(len(prof1d['ion'])):
-            prof1d_n['ion'][k]['density'] = copy.deepcopy(__zeros__)
-            for density in ['density_thermal','density_fast']:
-                if density in prof1d['ion'][k]:
-                    prof1d_n['ion'][k]['density'] += prof1d['ion'][k][density]
+            consistent_density(prof1d_n['ion'][k])
+
     return ods_n
 
+
 @add_to__ODS__
 def core_profiles_zeff(ods, update=True, use_electrons_density=False):
     '''
@@ -222,26 +246,27 @@ def core_profiles_zeff(ods, update=True, use_electrons_density=False):
     :return: updated ods
     '''
 
-    ods_z = core_profiles_densities(ods,update=update)
+    ods_z = core_profiles_densities(ods, update=update)
 
     for time_index in ods['core_profiles']['profiles_1d']:
         prof1d = ods['core_profiles']['profiles_1d'][time_index]
         prof1d_z = ods_z['core_profiles']['profiles_1d'][time_index]
 
-        Z2n = 0.*prof1d_z['grid']['rho_tor_norm']
-        Zn  = 0.*prof1d_z['grid']['rho_tor_norm']
+        Z2n = 0. * prof1d_z['grid']['rho_tor_norm']
+        Zn = 0. * prof1d_z['grid']['rho_tor_norm']
 
         for k in range(len(prof1d['ion'])):
-            Z = prof1d['ion'][k]['element'][0]['z_n'] # from old ODS
-            n = prof1d_z['ion'][k]['density']         # from new ODS
-            Z2n += n*Z**2
-            Zn  += n*Z
+            Z = prof1d['ion'][k]['element'][0]['z_n']  # from old ODS
+            n = prof1d_z['ion'][k]['density']  # from new ODS
+            Z2n += n * Z ** 2
+            Zn += n * Z
             if use_electrons_density:
-                prof1d_z['zeff'] = Z2n/prof1d_z['electrons']['density']
+                prof1d_z['zeff'] = Z2n / prof1d_z['electrons']['density']
             else:
-                prof1d_z['zeff'] = Z2n/Zn
+                prof1d_z['zeff'] = Z2n / Zn
     return ods_z
 
+
 @add_to__ODS__
 def current_from_eq(ods, time_index):
     """
@@ -283,6 +308,7 @@ def current_from_eq(ods, time_index):
 
     return
 
+
 @add_to__ODS__
 def core_profiles_currents(ods, time_index, rho_tor_norm,
                            j_actuator='default', j_bootstrap='default',
@@ -487,6 +513,102 @@ def core_profiles_currents(ods, time_index, rho_tor_norm,
 
     return
 
+
+@add_to__ODS__
+def wall_add(ods, machine=None):
+    '''
+    Add wall information to the ODS
+
+    :param ods: ODS to update in-place
+
+    :param machine: machine of which to load the wall (if None it is taken from ods['dataset_description.data_entry.machine'])
+    '''
+    if machine is None:
+        if 'machine' in ods['dataset_description.data_entry']:
+            machine = ods['dataset_description.data_entry.machine']
+        else:
+            raise LookupError('Could not figure out what machine wall to use: dataset_description.data_entry.machine is not set')
+
+    walls = {}
+    walls['iter'] = {'RLIM': [6.267, 7.283, 7.899, 8.306, 8.395, 8.27, 7.904, 7.4,
+                              6.587, 5.753, 4.904, 4.311, 4.126, 4.076, 4.046, 4.046,
+                              4.067, 4.097, 4.178, 3.9579, 4.0034, 4.1742, 4.3257, 4.4408,
+                              4.5066, 4.5157, 4.467, 4.4064, 4.4062, 4.3773, 4.3115, 4.2457,
+                              4.1799, 4.4918, 4.5687, 4.6456, 4.8215, 4.9982, 5.1496, 5.2529,
+                              5.2628, 5.2727, 5.565, 5.565, 5.565, 5.565, 5.572, 5.572,
+                              5.572, 5.572, 5.6008, 5.6842, 5.815, 5.9821, 6.171, 6.3655,
+                              6.267],
+                     'ZLIM': [-3.036, -2.247, -1.332, -0.411, 0.643, 1.691, 2.474,
+                              3.189, 3.904, 4.542, 4.722, 4.334, 3.592, 2.576,
+                              1.559, 0.543, -0.474, -1.49, -2.496, -2.5284, -2.5284,
+                              -2.5574, -2.6414, -2.7708, -2.931, -3.1039, -3.2701, -3.3943,
+                              -3.3948, -3.4699, -3.6048, -3.7397, -3.8747, -3.8992, -3.8176,
+                              -3.736, -3.699, -3.7314, -3.8282, -3.9752, -4.1144, -4.2536,
+                              -4.5459, -4.3926, -4.2394, -4.0862, -3.9861, -3.9856, -3.886,
+                              -3.885, -3.6924, -3.5165, -3.3723, -3.2722, -3.225, -3.2346,
+                              -3.036]}
+
+    if machine.lower() not in walls:
+        raise LookupError('OMAS wall information only available for: %s' % walls.keys())
+
+    ods['wall.description_2d.+.limiter.type.name'] = 'first_wall'
+    ods['wall.description_2d.-1.limiter.type.index'] = 0
+    ods['wall.description_2d.-1.limiter.type.description'] = 'first wall'
+    ods['wall.description_2d.-1.limiter.unit.0.outline.r'] = walls[machine.lower()]['RLIM']
+    ods['wall.description_2d.-1.limiter.unit.0.outline.z'] = walls[machine.lower()]['ZLIM']
+
+
+@add_to__ODS__
+def equilibrium_consistent(ods):
+    for time_index in ods['equilibrium.time_slice']:
+        eq = ods['equilibrium.time_slice'][time_index]
+        eq1d = ods['equilibrium.time_slice'][time_index]['profiles_1d']
+
+        psi = eq1d['psi']
+        psi_norm = abs(psi - psi[0]) / abs(psi[-1] - psi[0])
+        psirz = eq['profiles_2d.0.psi']
+        psirz_norm = abs(psirz - psi[0]) / abs(psi[-1] - psi[0])
+        fpol = eq1d['f']
+
+        # extend functions in PSI to be clamped at edge value when outside of PSI range (i.e. outside of LCFS)
+        ext_psi_norm_mesh = numpy.hstack((psi_norm[0] - 1E6, psi_norm, psi_norm[-1] + 1E6))
+
+        def ext_arr(inv):
+            return numpy.hstack((inv[0], inv, inv[-1]))
+
+        fpol = omas_interp1d(psirz_norm, ext_psi_norm_mesh, ext_arr(fpol))
+        Z, R = numpy.meshgrid(eq['profiles_2d.0.grid.dim2'], eq['profiles_2d.0.grid.dim1'])
+        eq['profiles_2d.0.r'] = R
+        eq['profiles_2d.0.z'] = Z
+        eq['profiles_2d.0.b_field_tor'] = fpol / R
+
+
+@add_to__ODS__
+def equilibrium_transpose_RZ(ods, flip_dims=False):
+    '''
+    Transpose 2D grid values for RZ grids under equilibrium.time_slice.:.profiles_2d.:.
+
+    :param ods: ODS to update in-place
+
+    :param flip_dims: whether to switch the equilibrium.time_slice.:.profiles_2d.:.grid.dim1 and dim1
+
+    :return: updated ods
+    '''
+    for time_index in ods['equilibrium.time_slice']:
+        for grid in ods['equilibrium.time_slice'][time_index]['profiles_2d']:
+            if ods['equilibrium.time_slice'][time_index]['profiles_2d'][grid]['grid_type.index'] == 1:
+                eq2D = ods['equilibrium.time_slice'][time_index]['profiles_2d'][grid]
+                for item in eq2D:
+                    if isinstance(eq2D[item], numpy.ndarray) and len(eq2D[item].shape) == 2:
+                        eq2D[item] = eq2D[item].T
+
+                if flip_dims:
+                    tmp = eq2D['grid.dim1']
+                    eq2D['grid.dim1'] = ed2D['grid.dim2']
+                    eq2D['grid.dim1'] = tmp
+    return ods
+
+
 @add_to__ALL__
 def transform_current(rho, JtoR=None, JparB=None, equilibrium=None, includes_bootstrap=False):
     """
@@ -535,6 +657,7 @@ def transform_current(rho, JtoR=None, JparB=None, equilibrium=None, includes_boo
 
     return Jout
 
+
 @add_to__ALL__
 def search_ion(ion_ods, label=None, Z=None, A=None, no_matches_raise_error=True, multiple_matches_raise_error=True):
     '''
@@ -555,7 +678,7 @@ def search_ion(ion_ods, label=None, Z=None, A=None, no_matches_raise_error=True,
     :return: dictionary with matching ions labels, each with list of matching ion elements
     '''
     if not ion_ods.location.endswith('.ion'):
-        raise (ValueError('ods location must end with `.ion`'))
+        raise ValueError('ods location must end with `.ion`')
     match = {}
     for ki in ion_ods:
         if label is None or (label is not None and 'label' in ion_ods[ki] and ion_ods[ki]['label'] == label):
@@ -572,11 +695,12 @@ def search_ion(ion_ods, label=None, Z=None, A=None, no_matches_raise_error=True,
             else:
                 match[ki] = []
     if multiple_matches_raise_error and (len(match) > 1 or len(match) == 1 and len(list(match.values())[0]) > 1):
-        raise (IndexError('Multiple ion match query: label=%s  Z=%s  A=%s' % (label, Z, A)))
+        raise IndexError('Multiple ion match query: label=%s  Z=%s  A=%s' % (label, Z, A))
     if no_matches_raise_error and len(match) == 0:
-        raise (IndexError('No ion match query: label=%s  Z=%s  A=%s' % (label, Z, A)))
+        raise IndexError('No ion match query: label=%s  Z=%s  A=%s' % (label, Z, A))
     return match
 
+
 @add_to__ALL__
 def search_in_array_structure(ods, conditions, no_matches_return=0, no_matches_raise_error=False, multiple_matches_raise_error=True):
     '''
@@ -600,7 +724,7 @@ def search_in_array_structure(ods, conditions, no_matches_return=0, no_matches_r
     '''
 
     if ods.omas_data is not None and not isinstance(ods.omas_data, list):
-        raise (Exception('ods location must be an array of structures'))
+        raise Exception('ods location must be an array of structures')
 
     if isinstance(conditions, ODS):
         conditions = conditions.flat()
@@ -625,14 +749,15 @@ def search_in_array_structure(ods, conditions, no_matches_return=0, no_matches_r
 
     if not len(match):
         if no_matches_raise_error:
-            raise (IndexError('no matches for conditions: %s' % conditions))
+            raise IndexError('no matches for conditions: %s' % conditions)
         match = [no_matches_return]
 
     if multiple_matches_raise_error and len(match) > 1:
-        raise (IndexError('multiple matches for conditions: %s' % conditions))
+        raise IndexError('multiple matches for conditions: %s' % conditions)
 
     return match
 
+
 @add_to__ALL__
 def define_cocos(cocos_ind):
     """
@@ -645,98 +770,99 @@ def define_cocos(cocos_ind):
     :return: dictionary with COCOS coefficients
     """
 
-    cocos=dict.fromkeys(['sigma_Bp', 'sigma_RpZ', 'sigma_rhotp', 'sign_q_pos', 'sign_pprime_pos', 'exp_Bp'])
+    cocos = dict.fromkeys(['sigma_Bp', 'sigma_RpZ', 'sigma_rhotp', 'sign_q_pos', 'sign_pprime_pos', 'exp_Bp'])
 
     # all multipliers shouldn't change input values if cocos_ind is None
     if cocos_ind is None:
-        cocos['exp_Bp']          = 0
-        cocos['sigma_Bp']        = +1
-        cocos['sigma_RpZ']       = +1
-        cocos['sigma_rhotp']     = +1
-        cocos['sign_q_pos']      = 0
+        cocos['exp_Bp'] = 0
+        cocos['sigma_Bp'] = +1
+        cocos['sigma_RpZ'] = +1
+        cocos['sigma_rhotp'] = +1
+        cocos['sign_q_pos'] = 0
         cocos['sign_pprime_pos'] = 0
         return cocos
 
     # if COCOS>=10, this should be 1
     cocos['exp_Bp'] = 0
-    if cocos_ind>=10:
+    if cocos_ind >= 10:
         cocos['exp_Bp'] = +1
 
-    if cocos_ind in [1,11]:
+    if cocos_ind in [1, 11]:
         # These cocos are for
         # (1)  psitbx(various options), Toray-GA
         # (11) ITER, Boozer
-        cocos['sigma_Bp']        = +1
-        cocos['sigma_RpZ']       = +1
-        cocos['sigma_rhotp']     = +1
-        cocos['sign_q_pos']      = +1
+        cocos['sigma_Bp'] = +1
+        cocos['sigma_RpZ'] = +1
+        cocos['sigma_rhotp'] = +1
+        cocos['sign_q_pos'] = +1
         cocos['sign_pprime_pos'] = -1
 
-    elif cocos_ind in [2,12,-12]:
+    elif cocos_ind in [2, 12, -12]:
         # These cocos are for
         # (2)  CHEASE, ONETWO, HintonHazeltine, LION, XTOR, MEUDAS, MARS, MARS-F
         # (12) GENE
         # (-12) ASTRA
-        cocos['sigma_Bp']        = +1
-        cocos['sigma_RpZ']       = -1
-        cocos['sigma_rhotp']     = +1
-        cocos['sign_q_pos']      = +1
+        cocos['sigma_Bp'] = +1
+        cocos['sigma_RpZ'] = -1
+        cocos['sigma_rhotp'] = +1
+        cocos['sign_q_pos'] = +1
         cocos['sign_pprime_pos'] = -1
 
-    elif cocos_ind in [3,13]:
+    elif cocos_ind in [3, 13]:
         # These cocos are for
         # (3) Freidberg*, CAXE and KINX*, GRAY, CQL3D^, CarMa, EFIT* with : ORB5, GBSwith : GT5D
         # (13) 	CLISTE, EQUAL, GEC, HELENA, EU ITM-TF up to end of 2011
-        cocos['sigma_Bp']        = -1
-        cocos['sigma_RpZ']       = +1
-        cocos['sigma_rhotp']     = -1
-        cocos['sign_q_pos']      = -1
+        cocos['sigma_Bp'] = -1
+        cocos['sigma_RpZ'] = +1
+        cocos['sigma_rhotp'] = -1
+        cocos['sign_q_pos'] = -1
         cocos['sign_pprime_pos'] = +1
 
-    elif cocos_ind in [4,14]:
-        #These cocos are for
-        cocos['sigma_Bp']        = -1
-        cocos['sigma_RpZ']       = -1
-        cocos['sigma_rhotp']     = -1
-        cocos['sign_q_pos']      = -1
+    elif cocos_ind in [4, 14]:
+        # These cocos are for
+        cocos['sigma_Bp'] = -1
+        cocos['sigma_RpZ'] = -1
+        cocos['sigma_rhotp'] = -1
+        cocos['sign_q_pos'] = -1
         cocos['sign_pprime_pos'] = +1
 
-    elif cocos_ind in [5,15]:
+    elif cocos_ind in [5, 15]:
         # These cocos are for
         # (5) TORBEAM, GENRAY^
-        cocos['sigma_Bp']        = +1
-        cocos['sigma_RpZ']       = +1
-        cocos['sigma_rhotp']     = -1
-        cocos['sign_q_pos']      = -1
+        cocos['sigma_Bp'] = +1
+        cocos['sigma_RpZ'] = +1
+        cocos['sigma_rhotp'] = -1
+        cocos['sign_q_pos'] = -1
         cocos['sign_pprime_pos'] = -1
 
-    elif cocos_ind in [6,16]:
-        #These cocos are for
-        cocos['sigma_Bp']        = +1
-        cocos['sigma_RpZ']       = -1
-        cocos['sigma_rhotp']     = -1
-        cocos['sign_q_pos']      = -1
+    elif cocos_ind in [6, 16]:
+        # These cocos are for
+        cocos['sigma_Bp'] = +1
+        cocos['sigma_RpZ'] = -1
+        cocos['sigma_rhotp'] = -1
+        cocos['sign_q_pos'] = -1
         cocos['sign_pprime_pos'] = -1
 
-    elif cocos_ind in [7,17]:
-        #These cocos are for
+    elif cocos_ind in [7, 17]:
+        # These cocos are for
         # (17) LIUQE*, psitbx(TCV standard output)
-        cocos['sigma_Bp']        = -1
-        cocos['sigma_RpZ']       = +1
-        cocos['sigma_rhotp']     = +1
-        cocos['sign_q_pos']      = +1
+        cocos['sigma_Bp'] = -1
+        cocos['sigma_RpZ'] = +1
+        cocos['sigma_rhotp'] = +1
+        cocos['sign_q_pos'] = +1
         cocos['sign_pprime_pos'] = +1
 
-    elif cocos_ind in [8,18]:
-        #These cocos are for
-        cocos['sigma_Bp']        = -1
-        cocos['sigma_RpZ']       = -1
-        cocos['sigma_rhotp']     = +1
-        cocos['sign_q_pos']      = +1
+    elif cocos_ind in [8, 18]:
+        # These cocos are for
+        cocos['sigma_Bp'] = -1
+        cocos['sigma_RpZ'] = -1
+        cocos['sigma_rhotp'] = +1
+        cocos['sign_q_pos'] = +1
         cocos['sign_pprime_pos'] = +1
 
     return cocos
 
+
 @add_to__ALL__
 def cocos_transform(cocosin_index, cocosout_index):
     """
@@ -753,21 +879,21 @@ def cocos_transform(cocosin_index, cocosout_index):
 
     # Don't transform if either cocos is undefined
     if (cocosin_index is None) or (cocosout_index is None):
-        printd("No COCOS tranformation for "+str(cocosin_index)+" to "+str(cocosout_index),topic='cocos')
-        sigma_Ip_eff    = 1
-        sigma_B0_eff    = 1
-        sigma_Bp_eff    = 1
-        exp_Bp_eff      = 0
+        printd("No COCOS tranformation for " + str(cocosin_index) + " to " + str(cocosout_index), topic='cocos')
+        sigma_Ip_eff = 1
+        sigma_B0_eff = 1
+        sigma_Bp_eff = 1
+        exp_Bp_eff = 0
         sigma_rhotp_eff = 1
     else:
-        printd("COCOS tranformation from "+str(cocosin_index)+" to "+str(cocosout_index),topic='cocos')
+        printd("COCOS tranformation from " + str(cocosin_index) + " to " + str(cocosout_index), topic='cocos')
         cocosin = define_cocos(cocosin_index)
         cocosout = define_cocos(cocosout_index)
 
-        sigma_Ip_eff    = cocosin['sigma_RpZ'] * cocosout['sigma_RpZ']
-        sigma_B0_eff    = cocosin['sigma_RpZ'] * cocosout['sigma_RpZ']
-        sigma_Bp_eff    = cocosin['sigma_Bp'] * cocosout['sigma_Bp']
-        exp_Bp_eff      = cocosout['exp_Bp'] - cocosin['exp_Bp']
+        sigma_Ip_eff = cocosin['sigma_RpZ'] * cocosout['sigma_RpZ']
+        sigma_B0_eff = cocosin['sigma_RpZ'] * cocosout['sigma_RpZ']
+        sigma_Bp_eff = cocosin['sigma_Bp'] * cocosout['sigma_Bp']
+        exp_Bp_eff = cocosout['exp_Bp'] - cocosin['exp_Bp']
         sigma_rhotp_eff = cocosin['sigma_rhotp'] * cocosout['sigma_rhotp']
 
     # Transform
@@ -791,6 +917,7 @@ def cocos_transform(cocosin_index, cocosout_index):
 
     return transforms
 
+
 @add_to__ALL__
 @contextmanager
 def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, **kw):
@@ -820,7 +947,7 @@ def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, **kw):
         coordsio = tmp
 
     if cocosio is not None and not isinstance(cocosio, int):
-        raise (ValueError('cocosio can only be an integer'))
+        raise ValueError('cocosio can only be an integer')
 
     # backup attributes
     bkp_cocosio = ods.cocosio
@@ -856,7 +983,7 @@ def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, **kw):
             setattr(ods, item, bkp_args[item])
 
 
-def generate_cocos_signals(structures=[], threshold=0):
+def generate_cocos_signals(structures=[], threshold=0, write=True, verbose=True):
     """
     This is a utility function for generating the omas_cocos.py Python file
 
@@ -866,13 +993,19 @@ def generate_cocos_signals(structures=[], threshold=0):
     * 0 is a reasonable threshold for catching signals that should have an associated COCOS transform
     * 10000 (or any high number) is a way to hide signals in omas_cocos.py that are unassigned
 
+    :param write: update omas_cocos.py file
+
+    :param verbose: print cocos signals to screen
+
     :return: dictionary structure with tally of score and reason for scoring for every entry
     """
     # units of entries currently in cocos_singals
     cocos_units = []
     for item in cocos_signals:
+        if cocos_signals[item] == '?':
+            continue
         info = omas_info_node(item)
-        if len(info): # info may have no length if nodes are deleted between IMAS versions
+        if len(info):  # info may have no length if nodes are deleted between IMAS versions
             units = info['units']
             if units not in cocos_units:
                 cocos_units.append(units)
@@ -896,19 +1029,43 @@ def generate_cocos_signals(structures=[], threshold=0):
     ods = ODS()
     out = {}
     text = []
-    csig=["'''List of automatic COCOS transformations\n\n-------\n'''",
-          '# COCOS signals candidates are generated by running utilities/generate_cocos_signals.py',
-          '# Running this script is useful to keep track of new signals that IMAS adds in new data structure releases',
-          '#',
-          '# In this file you are only allowed to edit/add entries to the `cocos_signals` dictionary',
-          '# The comments indicate `#[ADD_OR_DELETE_SUGGESTION]# MATCHING_SCORE # RATIONALE_FOR_ADD_OR_DELETE`',
-          '#',
-          '# Proceed as follows:',
-          '# 1. Edit transformations in this file (if a signal is missing, it can be added here)',
-          '# 2. Run `utilities/generate_cocos_signals.py` (which will update this same file)',
-          '# 3. Commit changes',
-          '',
-          'cocos_signals = {}']
+    csig = ["""
+'''List of automatic COCOS transformations
+
+-------
+'''
+# COCOS signals candidates are generated by running utilities/generate_cocos_signals.py
+# Running this script is useful to keep track of new signals that IMAS adds in new data structure releases
+#
+# In this file you are only allowed to edit/add entries to the `cocos_signals` dictionary
+# The comments indicate `#[ADD_OR_DELETE_SUGGESTION]# MATCHING_SCORE # RATIONALE_FOR_ADD_OR_DELETE`
+#
+# Proceed as follows:
+# 1. Edit transformations in this file (if a signal is missing, it can be added here)
+# 2. Run `utilities/generate_cocos_signals.py` (which will update this same file)
+# 3. Commit changes
+#
+# Valid transformations are defined in the `cocos_transform()` function and they are:
+#
+#        transforms = {}
+#        transforms['1/PSI'] = sigma_Ip_eff * sigma_Bp_eff / (2 * numpy.pi) ** exp_Bp_eff
+#        transforms['invPSI'] = transforms['1/PSI']
+#        transforms['dPSI'] = transforms['1/PSI']
+#        transforms['F_FPRIME'] = transforms['dPSI']
+#        transforms['PPRIME'] = transforms['dPSI']
+#        transforms['PSI'] = sigma_Ip_eff * sigma_Bp_eff * (2 * numpy.pi) ** exp_Bp_eff
+#        transforms['Q'] = sigma_Ip_eff * sigma_B0_eff * sigma_rhotp_eff
+#        transforms['TOR'] = sigma_B0_eff
+#        transforms['BT'] = transforms['TOR']
+#        transforms['IP'] = transforms['TOR']
+#        transforms['F'] = transforms['TOR']
+#        transforms['POL'] = sigma_B0_eff * sigma_rhotp_eff
+#        transforms['BP'] = transforms['POL']
+#        transforms[None] = 1
+#
+                
+cocos_signals = {}
+"""]
 
     # loop over structures
     for structure in structures:
@@ -920,12 +1077,12 @@ def generate_cocos_signals(structures=[], threshold=0):
         d = dict_structures(imas_version=omas_rcparams['default_imas_version'])
         m = 0
         # generate score and add reason for scoring
-        for item in sorted(list(_structures[(structure,omas_rcparams['default_imas_version'])].keys())):
+        for item in sorted(list(_structures[(structure, omas_rcparams['default_imas_version'])].keys())):
             item = i2o(item)
             item_ = item
-            if any([item.endswith(k) for k in [':.values',':.value',':.data']]):
+            if any([item.endswith(k) for k in [':.values', ':.value', ':.data']]):
                 item_ = l2o(p2l(item)[:-2])
-            elif any([item.endswith(k) for k in ['.values','.value','.data']]):
+            elif any([item.endswith(k) for k in ['.values', '.value', '.data']]):
                 item_ = l2o(p2l(item)[:-1])
             m = max(m, len(item))
             score = 0
@@ -941,10 +1098,10 @@ def generate_cocos_signals(structures=[], threshold=0):
                 elif units in [None, 's']:
                     out[structure].setdefault(-1, []).append((item, '[%s]' % units))
                     continue
-                elif any([(item_.endswith('.'+k) or item_.endswith('_'+k) or '.'+k+'.' in item) for k in
+                elif any([(item_.endswith('.' + k) or item_.endswith('_' + k) or '.' + k + '.' in item) for k in
                           ['chi_squared', 'standard_deviation', 'weight', 'coefficients', 'r', 'z', 'beta_tor',
                            'beta_pol', 'radial', 'rho_tor_norm', 'darea_drho_tor',
-                           'dvolume_drho_tor','ratio','fraction','rate','d','flux','v','b_field_max','width_tor']]):
+                           'dvolume_drho_tor', 'ratio', 'fraction', 'rate', 'd', 'flux', 'v', 'b_field_max', 'width_tor']]):
                     out[structure].setdefault(-1, []).append((item, p2l(item_)[-1]))
                     continue
                 elif any([k in documentation for k in ['always positive']]):
@@ -997,33 +1154,44 @@ def generate_cocos_signals(structures=[], threshold=0):
                 elif score < 0:
                     message = '#[DEL?]'
 
-                txt = ("cocos_signals['%s']='%s'" % (item, cocos_signals.get(item, '?'))).ljust(m + 20) + message + '# %f # %s' % (score, rationale)
+                txt = ("cocos_signals['%s']=%s" % (item, repr(cocos_signals.get(item, '?')))).ljust(m + 20) + message + '# %f # %s' % (score, rationale)
                 text.append(txt)
                 if score > threshold or (item in cocos_signals and cocos_signals[item] != '?'):
                     csig.append(txt)
 
-        # write omas_cocos.py
-        filename = os.path.abspath(str(os.path.dirname(__file__)) + '/omas_cocos.py')
-        with open(filename, 'w') as f:
-            f.write('\n'.join(csig))
-
         # print to screen (note that this prints ALL the entries, whereas omas_cocos.py only contains entries that score above a give threshold)
-        print('\n'.join(text) + '\n\n' + '-' * 20 + '\n\nUpdated ' + filename)
+        if verbose:
+            print('\n'.join(text) + '\n\n' + '-' * 20)
+
+    filename = os.path.abspath(str(os.path.dirname(__file__)) + '/omas_cocos.py')
+    if write:
+        # update omas_cocos.py
+        with open(filename, 'w') as f:
+            f.write(str('\n'.join(csig)))
+    else:
+        # check that omas_cocos.py file is up-to-date
+        with open(filename, 'r') as f:
+            tmp = str(f.read())
+        tmp1 = str('\n'.join(csig))
+        assert tmp == tmp1, 'COCOS signals are not up-to-date! Run `make cocos` to update the omas_cocos.py file.'
 
     return out
 
+
 # cocos_signals contains the IMAS locations and the corresponding `cocos_transform` function
 from .omas_cocos import cocos_signals as _cocos_signals
 
+
 # The CocosSignals class is just a dictionary that raises warnings when users access
 # entries that are likely to need a COCOS transformation, but do not have one.
 class CocosSignals(dict):
     def __getitem__(self, key):
         value = dict.__getitem__(self, key)
         if value == '?':
-            warnings.warn('`%s` may require defining its COCOS transform in omas/omas_cocos.py')
+            warnings.warn('`%s` may require defining its COCOS transform in omas/omas_cocos.py' % key)
         return value
 
+
 # cocos_signals is the actual dictionary
 cocos_signals = CocosSignals()
 cocos_signals.update(_cocos_signals)
