@@ -5,19 +5,13 @@
 
 from __future__ import print_function, division, unicode_literals
 
-import inspect
 from .omas_utils import *
 from .omas_physics import cocos_transform
 
-try:
-    import matplotlib
-    from matplotlib import pyplot
-except ImportError as _excp:
-    printe('OMAS plotting will not work! Error loading maplotlib: '+repr(_excp))
-
 __all__ = []
 __ods__ = []
 
+
 def add_to__ODS__(f):
     '''
     anything wrapped here will be available as a ODS method with name 'plot_'+f.__name__
@@ -35,127 +29,40 @@ def add_to__ALL__(f):
 # ================================
 # plotting helper functions
 # ================================
-def contourPaths(x, y, Z, levels, remove_boundary_points=False, smooth_factor=1):
-    """
-    :param x: 1D x coordinate
-
-    :param y: 1D y coordinate
-
-    :param Z: 2D data
-
-    :param levels: levels to trace
-
-    :param remove_boundary_points: remove traces at the boundary
-
-    :param smooth_factor: smooth contours by cranking up grid resolution. Requires scipy.
-
-    :return: list of segments
-    """
-    import matplotlib
-    try:
-        from scipy import ndimage
-    except ImportError:
-        printd('Warning: failed to import scipy in contourPaths. smoothing disabled.')
-        ndimage = None
-    if compare_version(matplotlib.__version__, '2.1') >= 0:
-        import matplotlib._contour as _contour
-    else:
-        from matplotlib import _cntr
-
-    sf = int(round(smooth_factor))
-    if sf > 1 and ndimage is not None:
-        x = ndimage.zoom(x, sf)
-        y = ndimage.zoom(y, sf)
-        Z = ndimage.zoom(Z, sf)
-
-    [X, Y] = numpy.meshgrid(x, y)
-    if compare_version(matplotlib.__version__, '2.1') >= 0:
-        contour_generator = _contour.QuadContourGenerator(X, Y, Z, None, True, 0)
-    else:
-        Cntr = matplotlib._cntr.Cntr(X, Y, Z)
-
-    allsegs = []
-    for level in levels:
-        if compare_version(matplotlib.__version__, '2.1') >= 0:
-            segs = contour_generator.create_contour(level)
-        else:
-            nlist = Cntr.trace(level)
-            nseg = len(nlist) // 2
-            segs = nlist[:nseg]
-        if not remove_boundary_points:
-            segs_ = segs
-        else:
-            segs_ = []
-            for segarray in segs:
-                x_ = segarray[:, 0]
-                y_ = segarray[:, 1]
-                valid = []
-                for i in range(len(x_) - 1):
-                    if numpy.isclose(x_[i], x_[i + 1]) and (
-                            numpy.isclose(x_[i], max(x)) or numpy.isclose(x_[i], min(x))):
-                        continue
-                    if numpy.isclose(y_[i], y_[i + 1]) and (
-                            numpy.isclose(y_[i], max(y)) or numpy.isclose(y_[i], min(y))):
-                        continue
-                    valid.append((x_[i], y_[i]))
-                    if i == len(x_):
-                        valid.append(x_[i + 1], y_[i + 1])
-                if len(valid):
-                    segs_.append(numpy.array(valid))
-
-        segs = map(matplotlib.path.Path, segs_)
-        allsegs.append(segs)
-    return allsegs
-
-
 class Uband(object):
     """
     This class wraps the line and PollyCollection(s) associated with a banded
     errorbar plot for use in the uband function.
 
-    It's methods are Line2D methods distributed to both the line and bands if
-    applicable, or just to the line alone otherwise.
-
     """
 
     def __init__(self, line, bands):
         """
         :param line: Line2D
             A line of the x,y nominal values
+
         :param bands: list of PolyCollections
-            The fill_between and/or fill_betweenx PollyCollections spanning the
-            std_devs of the x,y data.
+            The fill_between and/or fill_betweenx PollyCollections spanning the std_devs of the x,y data
 
         """
+        from matplotlib.cbook import flatten
         self.line = line  # matplotlib.lines.Line2D
-        self.bands = list(matplotlib.cbook.flatten([bands]))  # matplotlib.collections.PolyCollection(s)
+        self.bands = list(flatten([bands]))  # matplotlib.collections.PolyCollection(s)
 
+    def __getattr__(self, attr):
+        if attr in ['set_color', 'set_lw', 'set_linewidth', 'set_dashes', 'set_linestyle']:
+            def _band_line_method(self, method, *args, **kw):
+                """
+                Call the same method for line and band.
+                Returns Line2D method call result.
+                """
+                for band in self.bands:
+                    getattr(band, method)(*args, **kw)
+                return getattr(self.line, method)(*args, **kw)
 
-def _method_factory(self, key, bands=True):
-    """Add a method that calls the same method for line and band
-    or just for the line"""
-    if bands:
-        def method(self, *args, **kw):
-            """
-            Call the same method for line and band.
-            Returns Line2D method call result.
-            """
-            for band in self.bands:
-                getattr(band, key)(*args, **kw)
-            return getattr(self.line, key)(*args, **kw)
-    else:
-        def method(self, *args, **kw):
-            """Call the line method"""
-            return getattr(self.line, key)(*args, **kw)
-    return method
-
-
-for _name, _method in inspect.getmembers(matplotlib.lines.Line2D, predicate=inspect.ismethod):
-    if _name.startswith('_'):
-        continue
-    setattr(Uband, _name, _method_factory(Uband, _name,
-                                          bands=_name in ['set_color', 'set_lw', 'set_linewidth', 'set_dashes',
-                                                          'set_linestyle']))
+            return lambda method=attr, *args, **kw: _band_line_method(method, *args, **kw)
+        else:
+            return getattr(self.line, attr)
 
 
 def uband(x, y, ax=None, fill_kw={'alpha': 0.25}, **kw):
@@ -180,6 +87,8 @@ def uband(x, y, ax=None, fill_kw={'alpha': 0.25}, **kw):
 
     '''
 
+    from matplotlib import pyplot
+
     result = []
     if ax is None:
         ax = pyplot.gca()
@@ -278,7 +187,7 @@ def gas_filter(label, which_gas):
     return include
 
 
-def gas_arrow(ods, r, z, direction=None, snap_to=numpy.pi/4.0, ax=None, color=None, pad=1.0, **kw):
+def gas_arrow(ods, r, z, direction=None, snap_to=numpy.pi / 4.0, ax=None, color=None, pad=1.0, **kw):
     """
     Draws an arrow pointing in from the gas valve
     :param ods: ODS instance
@@ -303,13 +212,15 @@ def gas_arrow(ods, r, z, direction=None, snap_to=numpy.pi/4.0, ax=None, color=No
         Padding between arrow tip and specified (r,z)
     """
 
+    from matplotlib import pyplot
+
     def pick_direction():
         """Guesses the direction for the arrow (from injector toward machine) in case you don't know"""
         dr = ods['equilibrium']['time_slice'][0]['global_quantities']['magnetic_axis']['r'] - r
         dz = ods['equilibrium']['time_slice'][0]['global_quantities']['magnetic_axis']['z'] - z
         theta = numpy.arctan2(dz, -dr)
         if snap_to > 0:
-            theta = snap_to * round(theta/snap_to)
+            theta = snap_to * round(theta / snap_to)
         return theta
 
     if direction is None:
@@ -320,22 +231,22 @@ def gas_arrow(ods, r, z, direction=None, snap_to=numpy.pi/4.0, ax=None, color=No
     if ax is None:
         ax = pyplot.gca()
 
-    shaft_len = 3.5 * (1+pad)/2.
+    shaft_len = 3.5 * (1 + pad) / 2.
 
-    da = numpy.pi/10  # Angular half width of the arrow head
+    da = numpy.pi / 10  # Angular half width of the arrow head
     x0 = numpy.cos(-direction) * pad
     y0 = numpy.sin(-direction) * pad
     head_mark = [
         (x0, y0),
-        (x0+numpy.cos(-direction+da), y0+numpy.sin(-direction+da)),
-        (x0+numpy.cos(-direction), y0+numpy.sin(-direction)),
-        (x0+shaft_len*numpy.cos(-direction), y0+shaft_len*numpy.sin(-direction)),
-        (x0+numpy.cos(-direction), y0+numpy.sin(-direction)),
-        (x0+numpy.cos(-direction-da), y0+numpy.sin(-direction-da)),
+        (x0 + numpy.cos(-direction + da), y0 + numpy.sin(-direction + da)),
+        (x0 + numpy.cos(-direction), y0 + numpy.sin(-direction)),
+        (x0 + shaft_len * numpy.cos(-direction), y0 + shaft_len * numpy.sin(-direction)),
+        (x0 + numpy.cos(-direction), y0 + numpy.sin(-direction)),
+        (x0 + numpy.cos(-direction - da), y0 + numpy.sin(-direction - da)),
     ]
 
     kw.pop('marker', None)  # Ignore this
-    return ax.plot(r, z, marker=head_mark, color=color, markersize=100*(pad+shaft_len)/5, **kw)
+    return ax.plot(r, z, marker=head_mark, color=color, markersize=100 * (pad + shaft_len) / 5, **kw)
 
 
 def geo_type_lookup(geometry_type, subsys, imas_version=omas_rcparams['default_imas_version'], reverse=False):
@@ -390,7 +301,7 @@ def geo_type_lookup(geometry_type, subsys, imas_version=omas_rcparams['default_i
 # ODSs' plotting methods
 # ================================
 @add_to__ODS__
-def equilibrium_CX(ods, time_index=0, contour_smooth=3, levels=numpy.r_[0.1:10:0.1], ax=None, **kw):
+def equilibrium_CX(ods, time_index=0, levels=numpy.r_[0.1:10:0.1], ax=None, **kw):
     """
     Plot equilibrium cross-section
     as per `ods['equilibrium']['time_slice'][time_index]`
@@ -399,8 +310,6 @@ def equilibrium_CX(ods, time_index=0, contour_smooth=3, levels=numpy.r_[0.1:10:0
 
     :param time_index: time slice to plot
 
-    :param contour_smooth: Provides smoother contours by up-sampling first if >= 1 after rounding to nearest int.
-
     :param levels: list of sorted numeric values to pass to 2D plot as contour levels
 
     :param ax: axes to plot in (active axes is generated if `ax is None`)
@@ -409,12 +318,13 @@ def equilibrium_CX(ods, time_index=0, contour_smooth=3, levels=numpy.r_[0.1:10:0
 
     :return: axes
     """
+
+    import matplotlib
+    from matplotlib import pyplot
+
     if ax is None:
         ax = pyplot.gca()
 
-    label = kw.pop('label', '')  # Withhold this from all plots except the boundary to avoid spamming legend
-    kw.setdefault('linewidth', 1)
-
     wall = None
     eq = ods['equilibrium']['time_slice'][time_index]
     if 'wall' in ods:
@@ -423,6 +333,20 @@ def equilibrium_CX(ods, time_index=0, contour_smooth=3, levels=numpy.r_[0.1:10:0
         elif 0 in ods['wall']['description_2d']:
             wall = ods['wall']['description_2d'][0]['limiter']['unit']
 
+    # plotting style
+    kw.setdefault('linewidth', 1)
+    label = kw.pop('label', '')
+    kw1 = copy.deepcopy(kw)
+    kw1['linewidth'] = kw['linewidth'] + 1
+
+    # boundary
+    ax.plot(eq['boundary']['outline']['r'], eq['boundary']['outline']['z'], label=label, **kw1)
+    kw1.setdefault('color', ax.lines[-1].get_color())
+
+    # axis
+    if 'global_quantities.magnetic_axis.r' in eq and 'global_quantities.magnetic_axis.z':
+        ax.plot(eq['global_quantities']['magnetic_axis']['r'], eq['global_quantities']['magnetic_axis']['z'], '+', **kw1)
+
     # first try to plot as function of `rho` and fallback on `psi`
     if 'phi' in eq['profiles_2d'][0] and 'phi' in eq['profiles_1d']:
         value2D = numpy.sqrt(abs(eq['profiles_2d'][0]['phi']))
@@ -432,33 +356,26 @@ def equilibrium_CX(ods, time_index=0, contour_smooth=3, levels=numpy.r_[0.1:10:0
         value1D = eq['profiles_1d']['psi']
     value2D = (value2D - min(value1D)) / (max(value1D) - min(value1D))
 
+    # wall clipping
+    if wall is not None:
+        path = matplotlib.path.Path(numpy.transpose(numpy.array([wall[0]['outline']['r'], wall[0]['outline']['z']])))
+        wall_path = matplotlib.patches.PathPatch(path, facecolor='none')
+        ax.add_patch(wall_path)
+
     # contours
-    line = numpy.array([numpy.nan, numpy.nan])
-    for item1 in contourPaths(eq['profiles_2d'][0]['grid']['dim1'], eq['profiles_2d'][0]['grid']['dim2'], value2D.T,
-                              levels, smooth_factor=contour_smooth):
-        for item in item1:
-            line = numpy.vstack((line, item.vertices, numpy.array([numpy.nan, numpy.nan])))
+    if 'r' in eq['profiles_2d'][0] and 'z' in eq['profiles_2d'][0]:
+        R = eq['profiles_2d'][0]['r']
+        Z = eq['profiles_2d'][0]['z']
+    else:
+        Z, R = numpy.meshgrid(eq['profiles_2d'][0]['grid']['dim2'], eq['profiles_2d'][0]['grid']['dim1'])
+    kw.setdefault('colors', kw1['color'])
+    kw['linewidths'] = kw.pop('linewidth')
+    CS = ax.contour(R, Z, value2D, levels, **kw)
 
     # internal flux surfaces w/ or w/o masking
     if wall is not None:
-        path = matplotlib.path.Path(numpy.transpose(numpy.array([wall[0]['outline']['r'], wall[0]['outline']['z']])))
-        patch = matplotlib.patches.PathPatch(path, facecolor='none')
-        ax.add_patch(patch)
-        pyplot.plot(line[:, 0], line[:, 1], **kw)
-        ax.lines[-1].set_clip_path(patch)
-    else:
-        pyplot.plot(line[:, 0], line[:, 1], **kw)
-
-    # plotting style
-    kw1 = copy.deepcopy(kw)
-    kw1['linewidth'] = kw['linewidth'] + 1
-    kw1.setdefault('color', ax.lines[-1].get_color())
-
-    # boundary
-    ax.plot(eq['boundary']['outline']['r'], eq['boundary']['outline']['z'], label=label, **kw1)
-
-    # axis
-    ax.plot(eq['global_quantities']['magnetic_axis']['r'], eq['global_quantities']['magnetic_axis']['z'], '+', **kw1)
+        for collection in CS.collections:
+            collection.set_clip_path(wall_path)
 
     # wall
     if wall is not None:
@@ -492,10 +409,13 @@ def equilibrium_summary(ods, time_index=0, fig=None, **kw):
 
     :return: figure handler
     """
+
+    from matplotlib import pyplot
+
     if fig is None:
         fig = pyplot.figure()
 
-    ax = pyplot.subplot(1, 3, 1)
+    ax = fig.add_subplot(1, 3, 1)
     ax = equilibrium_CX(ods, time_index=time_index, ax=ax, **kw)
     eq = ods['equilibrium']['time_slice'][time_index]
 
@@ -509,7 +429,7 @@ def equilibrium_summary(ods, time_index=0, fig=None, **kw):
     x = (x - min(x)) / (max(x) - min(x))
 
     # pressure
-    ax = pyplot.subplot(2, 3, 2)
+    ax = fig.add_subplot(2, 3, 2)
     ax.plot(x, eq['profiles_1d']['pressure'], **kw)
     kw.setdefault('color', ax.lines[-1].get_color())
     ax.set_title('$\,$ Pressure')
@@ -568,6 +488,9 @@ def core_profiles_summary(ods, time_index=0, fig=None, combine_dens_temps=True,
 
     :return: figure handler
     """
+
+    from matplotlib import pyplot
+
     if fig is None:
         fig = pyplot.figure()
 
@@ -597,9 +520,9 @@ def core_profiles_summary(ods, time_index=0, fig=None, combine_dens_temps=True,
             if item + '.density' + therm_fast in prof1d:
                 if combine_dens_temps:
                     if ax0 is None:
-                        ax = ax0 = pyplot.subplot(1, 2, 1)
+                        ax = ax0 = fig.add_subplot(1, 2, 1)
                 else:
-                    ax = ax0 = pyplot.subplot(r, 2, (2 * k) + 1, sharex=ax)
+                    ax = ax0 = fig.add_subplot(r, 2, (2 * k) + 1, sharex=ax)
                 uband(x, prof1d[density], label=names[k] + therm_fast_name, ax=ax0, **kw)
                 if k == len(prof1d['ion']):
                     ax0.set_xlabel('$\\rho$')
@@ -614,9 +537,9 @@ def core_profiles_summary(ods, time_index=0, fig=None, combine_dens_temps=True,
         if item + '.temperature' in prof1d:
             if combine_dens_temps:
                 if ax1 is None:
-                    ax = ax1 = pyplot.subplot(1, 2, 2, sharex=ax)
+                    ax = ax1 = fig.add_subplot(1, 2, 2, sharex=ax)
             else:
-                ax = ax1 = pyplot.subplot(r, 2, (2 * k) + 2, sharex=ax)
+                ax = ax1 = fig.add_subplot(r, 2, (2 * k) + 2, sharex=ax)
             uband(x, prof1d[item + '.temperature'], label=names[k], ax=ax1, **kw)
             if k == len(prof1d['ion']):
                 ax1.set_xlabel('$\\rho$')
@@ -646,6 +569,9 @@ def core_profiles_pressures(ods, time_index=0, ax=None, **kw):
 
     :return: axes handler
     """
+
+    from matplotlib import pyplot
+
     if ax is None:
         ax = pyplot.gca()
 
@@ -720,6 +646,9 @@ def overlay(ods, ax=None, allow_autoscale=True, debug_all_plots=False, **kw):
 
             * Additional keywords are passed to the function that does the drawing; usually matplotlib.axes.Axes.plot().
     """
+
+    from matplotlib import pyplot
+
     if ax is None:
         ax = pyplot.gca()
 
@@ -786,6 +715,9 @@ def gas_injection_overlay(
         * Remaining keywords are passed to plot call for drawing markers at the gas locations.
 
     """
+
+    from matplotlib import pyplot
+
     # Make sure there is something to plot or else just give up and return
     npipes = get_channel_count(
         ods, 'gas_injection', check_loc='gas_injection.pipe.0.exit_position.r', test_checker='checker > 0',
@@ -820,7 +752,7 @@ def gas_injection_overlay(
 
             if angle_not_in_pipe_name:
                 try:
-                    label += ' ({:0d})'.format(int(round(pipe['exit_position']['phi']*180/numpy.pi)))
+                    label += ' ({:0d})'.format(int(round(pipe['exit_position']['phi'] * 180 / numpy.pi)))
                 except (TypeError, ValueError):
                     pass
     try:
@@ -840,7 +772,7 @@ def gas_injection_overlay(
     colors *= int(numpy.ceil(len(locations) / float(len(colors))))  # Make sure the list is long enough.
     for i, loc in enumerate(locations):
         r, z = numpy.array(loc.split('_')).astype(float)
-        label = '{spacer:}\n{spacer:}'.format(spacer=' '*label_spacer).join([''] + locations[loc] + [''])
+        label = '{spacer:}\n{spacer:}'.format(spacer=' ' * label_spacer).join([''] + locations[loc] + [''])
         if draw_arrow:
             kw.update(draw_arrow if isinstance(draw_arrow, dict) else {})
             gas_mark = gas_arrow(ods, r, z, ax=ax, color=colors[i], **kw)
@@ -849,7 +781,7 @@ def gas_injection_overlay(
         kw.pop('label', None)  # Prevent label from being applied every time through the loop to avoid spammy legend
         if (labelevery > 0) and ((i % labelevery) == 0):
             va = ['top', 'bottom'][int(z > 0)]
-            label = '\n'*label_spacer + label if va == 'top' else label + '\n'*label_spacer
+            label = '\n' * label_spacer + label if va == 'top' else label + '\n' * label_spacer
             ax.text(r, z, label, color=gas_mark[0].get_color(),
                     va=va, ha=['left', 'right'][int(r < rsplit)], fontsize=notesize)
     return
@@ -874,6 +806,10 @@ def pf_active_overlay(ods, ax=None, **kw):
             Hint: you may want to set facecolor instead of just color
     """
     # Make sure there is something to plot or else just give up and return
+
+    import matplotlib
+    from matplotlib import pyplot
+
     nc = get_channel_count(
         ods, 'pf_active', check_loc='pf_active.coil.0.element.0.geometry.geometry_type', channels_name='coil',
         test_checker='checker > -1')
@@ -974,6 +910,9 @@ def magnetics_overlay(
 
         * Remaining keywords are passed to plot call
     """
+
+    from matplotlib import pyplot
+
     # Make sure there is something to plot or else just give up and return
     nbp = get_channel_count(
         ods, 'magnetics', check_loc='magnetics.bpol_probe.0.position.r', channels_name='bpol_probe',
@@ -993,7 +932,7 @@ def magnetics_overlay(
     kw.pop('marker', None)
     kw.setdefault('linestyle', ' ')
     labelevery = kw.pop('labelevery', 0)
-    mask = kw.pop('mask', numpy.ones(nbp+nfl, bool))
+    mask = kw.pop('mask', numpy.ones(nbp + nfl, bool))
     notesize = kw.pop('notesize', 'xx-small')
 
     def show_mag(n, topname, posroot, label, color_, marker, mask_):
@@ -1030,6 +969,9 @@ def interferometer_overlay(ods, ax=None, **kw):
 
         * Remaining keywords are passed to plot call
     """
+
+    from matplotlib import pyplot
+
     # Make sure there is something to plot or else just give up and return
     nc = get_channel_count(
         ods, 'interferometer', check_loc='interferometer.channel.0.line_of_sight.first_point.r',
@@ -1074,6 +1016,9 @@ def thomson_scattering_overlay(ods, ax=None, **kw):
 
         * Remaining keywords are passed to plot call
     """
+
+    from matplotlib import pyplot
+
     # Make sure there is something to plot or else just give up and return
     nc = get_channel_count(
         ods, 'thomson_scattering', check_loc='thomson_scattering.channel.0.position.r', test_checker='checker > 0')
@@ -1131,6 +1076,9 @@ def charge_exchange_overlay(ods, ax=None, which_pos='closest', **kw):
 
         * Remaining keywords are passed to plot call
     """
+
+    from matplotlib import pyplot
+
     # Make sure there is something to plot or else just give up and return
     nc = get_channel_count(
         ods, 'charge_exchange', check_loc='charge_exchange.channel.0.position.r.data', test_checker='any(checker > 0)')
@@ -1219,6 +1167,9 @@ def bolometer_overlay(ods, ax=None, reset_fan_color=True, colors=None, **kw):
 
         * Remaining keywords are passed to plot call for drawing markers at the gas locations.
     """
+
+    from matplotlib import pyplot
+
     # Make sure there is something to plot or else just give up and return
     nc = get_channel_count(
         ods, 'bolometer', check_loc='bolometer.channel.0.line_of_sight.first_point.r', test_checker='checker > 0')
@@ -1238,7 +1189,7 @@ def bolometer_overlay(ods, ax=None, reset_fan_color=True, colors=None, **kw):
     ncm = len(r1)
 
     if colors is None:
-        colors = [kw.pop('color', None)]*nc
+        colors = [kw.pop('color', None)] * nc
     else:
         colors *= nc  # Just make sure that this will always be long enough.
     ci = 0
@@ -1248,7 +1199,7 @@ def bolometer_overlay(ods, ax=None, reset_fan_color=True, colors=None, **kw):
     labelevery = kw.pop('labelevery', 2)
     notesize = kw.pop('notesize', 'xx-small')
     for i in range(ncm):
-        if (i > 0) and (bolo_id[i][0] != bolo_id[i-1][0]) and reset_fan_color:
+        if (i > 0) and (bolo_id[i][0] != bolo_id[i - 1][0]) and reset_fan_color:
             ci += 1
             color = colors[ci]  # Allow color to reset when changing fans
             new_label = True
@@ -1264,12 +1215,56 @@ def bolometer_overlay(ods, ax=None, reset_fan_color=True, colors=None, **kw):
             ax.text(r2[i], z2[i], '{}{}'.format(['\n', ''][int(z1[i] > 0)], bolo_id[i]), color=color,
                     ha=['right', 'left'][int(z1[i] > 0)], va='top', fontsize=notesize)
 
+
+@add_to__ODS__
+def summary(ods, fig=None, quantity=None):
+    '''
+    Plot summary time traces. Internally makes use of plot_quantity method.
+
+    :param ods: input ods
+
+    :param fig: figure to plot in (a new figure is generated if `fig is None`)
+
+    :param quantity: if None plot all time-dependent global_quantities. Else a list of strings with global quantities to plot
+
+    :return: figure handler
+    '''
+
+    from matplotlib import pyplot
+
+    if fig is None:
+        fig = pyplot.figure()
+    if quantity is None:
+        quantity = ods['summary.global_quantities']
+
+    # two passes, one for counting number of plots the second for actual plotting
+    n = 0
+    for step in ['count', 'plot']:
+        k = 0
+        for q in quantity:
+            if 'value' in ods['summary.global_quantities'][q] and isinstance(ods['summary.global_quantities'][q]['value'], numpy.ndarray):
+                if step == 'count':
+                    n += 1
+                k += 1
+                if step == 'plot':
+                    r = int(numpy.sqrt(n + 1))
+                    c = int(numpy.ceil(n / numpy.sqrt(n)))
+                    if k == 1:
+                        ax = ax0 = fig.add_subplot(r, c, k)
+                    else:
+                        ax = fig.add_subplot(r, c, k, sharex=ax0)
+                    ax.set_title(q)
+                    ods.plot_quantity('summary.global_quantities.%s.value' % q, label=q, ax=ax, xlabel=['', None][int(k > (n - c))])
+    return fig
+
+
 latexit = {}
 latexit['rho_tor_norm'] = '$\\rho$'
 latexit['zeff'] = '$Z_{\\rm eff}$'
 latexit['m^-3'] = '$m^{-3}$'
 latexit['psi'] = '$\\psi$'
 
+
 @add_to__ODS__
 def quantity(ods, key,
              yname=None, xname=None,
@@ -1314,6 +1309,9 @@ def quantity(ods, key,
     :return: axes instance
 
     '''
+
+    from matplotlib import pyplot
+
     # handle regular expressions
     key = ods.search_paths(key, 1, '@')[0]
 
@@ -1355,3 +1353,8 @@ def quantity(ods, key,
     ax.set_xlabel(xlabel)
     ax.set_ylabel(ylabel)
     return ax
+
+
+# this test is here to prevent
+if 'matplotlib' in locals() or 'pyplot' in locals() or 'plt' in locals():
+    raise Exception('Do not import matplotlib at the top level of %s' % os.path.split(__file__)[1])
