@@ -26,15 +26,14 @@ formatwarning_orig = warnings.formatwarning
 warnings.formatwarning = lambda message, category, filename, lineno, line=None: \
     formatwarning_orig(message, category, filename, lineno, line='')
 
-# pint
-try:
+# pint: avoid loading pint upfront since it can be slow and it is not always used
+ureg = []
+if False:
     import pint
-    from pint import UnitRegistry
-    ureg = UnitRegistry()
-except ImportError:
-    pint = None
-    ureg = None
-    warnings.warn('pint Python library not found')
+
+    ureg.append(pint.UnitRegistry())
+else:
+    ureg.append(None)
 
 # uncertainties
 import uncertainties
@@ -42,28 +41,46 @@ import uncertainties.unumpy as unumpy
 from uncertainties.unumpy import nominal_values, std_devs, uarray
 from uncertainties import ufloat
 
-# xarrays
+# xarrays: avoid loading xarrays upfront since it can be slow and it is not always used
 import xarray
 
 # Python3/2 import differences
 if sys.version_info < (3, 0):
     import cPickle as pickle
+
+
+    def b2s(string):
+        return string
+
 else:
     basestring = str
     unicode = str
     import pickle
 
-    _orig_pickle_loads=pickle.loads
-    def _pickle_loads_python2compatible(*args,**kw):
-        kw.setdefault('encoding','latin1')
-        return _orig_pickle_loads(*args,**kw)
-    pickle.loads=_pickle_loads_python2compatible
+    _orig_pickle_loads = pickle.loads
+
+
+    def _pickle_loads_python2compatible(*args, **kw):
+        kw.setdefault('encoding', 'latin1')
+        return _orig_pickle_loads(*args, **kw)
+
+
+    pickle.loads = _pickle_loads_python2compatible
+
+    _orig_pickle_load = pickle.load
+
+
+    def _pickle_load_python2compatible(*args, **kw):
+        kw.setdefault('encoding', 'latin1')
+        return _orig_pickle_load(*args, **kw)
+
+
+    pickle.load = _pickle_load_python2compatible
+
+
+    def b2s(bytes):
+        return bytes.decode("utf-8")
 
-    _orig_pickle_load=pickle.load
-    def _pickle_load_python2compatible(*args,**kw):
-        kw.setdefault('encoding','latin1')
-        return _orig_pickle_load(*args,**kw)
-    pickle.load=_pickle_load_python2compatible
 
 # --------------------------------------------
 # configuration of directories and IMAS infos
@@ -73,13 +90,18 @@ class IMAS_json_dir(unicode):
     directory where the JSON data structures for the different versions of IMAS are stored
     '''
     pass
+
+
 imas_json_dir = IMAS_json_dir(os.path.abspath(str(os.path.dirname(__file__)) + '/imas_structures/'))
 
+
 class IMAS_versions(OrderedDict):
     '''
     dictionary with list of IMAS version and their sub-folder name in the imas_json_dir
     '''
     pass
+
+
 imas_versions = IMAS_versions()
 # first `develop/3` and other branches
 for _item in list(map(lambda x: os.path.basename(x), sorted(glob.glob(imas_json_dir + os.sep + '*')))):
@@ -99,13 +121,6 @@ else:
         # IndexError will occur if `imas_json_dir` is empty: we must allow going forward, at least to build_json_structures
         _default_imas_version = ''
 
-# --------------------------------------------
-# configuration of directories and ITM infos
-# --------------------------------------------
-if 'OMAS_DATAVERSION_TAG' in os.environ:
-    _default_itm_version = os.environ['OMAS_DATAVERSION_TAG']
-else:
-    _default_itm_version = '4.10b_rc'
 
 # --------------------------------------------
 # rcparams
@@ -116,6 +131,7 @@ class OMAS_rc_params(dict):
     '''
     pass
 
+
 omas_rcparams = OMAS_rc_params()
 omas_rcparams.update({
     'cocos': 11,
@@ -124,30 +140,27 @@ omas_rcparams.update({
     'consistency_check': True,
     'dynamic_path_creation': True,
     'unitsio': False,
-    'tmp_imas_dir': os.environ.get('OMAS_TMP_DIR',
-                                   os.sep.join(
-                                       [tempfile.gettempdir(), 'OMAS_TMP_DIR'])),
-    'fake_imas_dir': os.environ.get('OMAS_FAKE_IMAS_DIR',
-                                    os.sep.join(
-                                        [os.environ.get('HOME', tempfile.gettempdir()), 'tmp', 'OMAS_FAKE_IMAS_DIR'])),
+    'tmp_imas_dir': os.environ.get('OMAS_TMP_DIR', os.sep.join([tempfile.gettempdir(),
+                                                                os.environ.get('USER', 'dummy_user'),
+                                                                'OMAS_TMP_DIR'])),
+    'fake_imas_dir': os.environ.get('OMAS_FAKE_IMAS_DIR', os.sep.join([os.environ.get('HOME', tempfile.gettempdir()),
+                                                                       'tmp',
+                                                                       'OMAS_FAKE_IMAS_DIR'])),
     'allow_fake_imas_fallback': bool(int(os.environ.get('OMAS_ALLOW_FAKE_IMAS_FALLBACK', '0'))),
-    'fake_itm_dir': os.environ.get('OMAS_FAKE_ITM_DIR',
-                                   os.sep.join(
-                                       [os.environ.get('HOME', tempfile.gettempdir()), 'tmp', 'OMAS_FAKE_ITM_DIR'])),
-    'allow_fake_itm_fallback': bool(int(os.environ.get('OMAS_ALLOW_FAKE_ITM_FALLBACK', '0'))),
-    'default_imas_version': _default_imas_version,
-    'default_itm_version': _default_itm_version
+    'default_imas_version': _default_imas_version
 })
 
+
 @contextmanager
 def rcparams_environment(**kw):
-    old_omas_rcparams=omas_rcparams.copy()
+    old_omas_rcparams = omas_rcparams.copy()
     omas_rcparams.update(kw)
     try:
         yield omas_rcparams
     finally:
         omas_rcparams.update(old_omas_rcparams)
 
+
 # --------------------------------------------
 # additional data structures
 # --------------------------------------------
@@ -179,4 +192,4 @@ add_datastructures['info'] = {
         "data_type": "INT_0D",
         "description": "run number"
     }
-}
\ No newline at end of file
+}
