@@ -8,6 +8,7 @@ from __future__ import print_function, division, unicode_literals
 from .omas_setup import *
 import sys
 
+
 # --------------------------------------------
 # ODS utilities
 # --------------------------------------------
@@ -29,22 +30,22 @@ def different_ods(ods1, ods2):
     differences = []
     for k in k1.difference(k2):
         if not k.startswith('info.'):
-            differences.append( 'DIFF: key `%s` missing in 2nd ods' % k )
+            differences.append('DIFF: key `%s` missing in 2nd ods' % k)
     for k in k2.difference(k1):
         if not k.startswith('info.'):
-            differences.append( 'DIFF: key `%s` missing in 1st ods' % k )
+            differences.append('DIFF: key `%s` missing in 1st ods' % k)
     for k in k1.intersection(k2):
         if isinstance(ods1[k], basestring) and isinstance(ods2[k], basestring):
             if ods1[k] != ods2[k]:
-                differences.append( 'DIFF: `%s` differ in value' % k )
+                differences.append('DIFF: `%s` differ in value' % k)
         elif type(ods1[k]) != type(ods2[k]):
-            differences.append( 'DIFF: `%s` differ in type (%s,%s)' % (k, type(ods1[k]), type(ods2[k])) )
+            differences.append('DIFF: `%s` differ in type (%s,%s)' % (k, type(ods1[k]), type(ods2[k])))
         elif numpy.atleast_1d(is_uncertain(ods1[k])).any() or numpy.atleast_1d(is_uncertain(ods2[k])).any():
             if not numpy.allclose(nominal_values(ods1[k]), nominal_values(ods2[k]), equal_nan=True) or not numpy.allclose(std_devs(ods1[k]), std_devs(ods2[k]), equal_nan=True):
-                differences.append( 'DIFF: `%s` differ in value' % k )
+                differences.append('DIFF: `%s` differ in value' % k)
         else:
             if not numpy.allclose(ods1[k], ods2[k], equal_nan=True):
-                differences.append( 'DIFF: `%s` differ in value' % k )
+                differences.append('DIFF: `%s` differ in value' % k)
     if len(differences):
         return differences
     else:
@@ -70,7 +71,10 @@ def printd(*objects, **kw):
         dump = True
         topic_selected = re.sub('_dump$', '', topic_selected)
     if topic_selected and (topic_selected == '*' or topic_selected in topic or '*' in topic):
-        printe(*objects, **kw)
+        if eval(os.environ.get('OMAS_DEBUG_STDOUT', '1')):
+            print(*objects, **kw)
+        else:
+            printe(*objects, **kw)
         if dump:
             fb = StringIO()
             print(*objects[1:], file=fb)
@@ -98,15 +102,16 @@ def is_uncertain(var):
     :return: True if variable is instance of uncertainties or
              array of shape var with elements indicating uncertainty
     '''
+
     def _uncertain_check(x):
         return isinstance(x, uncertainties.core.AffineScalarFunc)
 
-    if isinstance(var,basestring):
+    if isinstance(var, basestring):
         return False
     elif numpy.iterable(var):
-        tmp=numpy.array(var).flat
-        tmp=numpy.array(list(map(_uncertain_check, tmp)))
-        return numpy.reshape(tmp,numpy.array(var).shape)
+        tmp = numpy.array(var).flat
+        tmp = numpy.array(list(map(_uncertain_check, tmp)))
+        return numpy.reshape(tmp, numpy.array(var).shape)
     else:
         return _uncertain_check(var)
 
@@ -120,7 +125,7 @@ def is_numeric(value):
     :return: True/False
     """
     try:
-        0+value
+        0 + value
         return True
     except TypeError:
         return False
@@ -136,6 +141,8 @@ def omas_interp1d(x, xp, fp, left=None, right=None, period=None):
         index = numpy.argsort(xp)
         return numpy.interp(x, xp[index], fp[index], left=left, right=right, period=period)
     return numpy.interp(x, xp, fp, left=left, right=right, period=period)
+
+
 omas_interp1d.__doc__ += numpy.interp.__doc__
 
 
@@ -161,18 +168,18 @@ def json_dumper(obj, objects_encode=True):
         if isinstance(obj, numpy.ndarray):
             return obj.tolist()
         elif isinstance(obj, numpy.generic):
-            return numpy.asscalar(obj)
+            return obj.item()
         else:
             return obj.toJSON()
 
     else:
-        tmp=is_uncertain(obj)
+        tmp = is_uncertain(obj)
         if numpy.any(numpy.atleast_1d(tmp)):
             if not len(numpy.array(tmp).shape):
                 return dict(__ufloat__=nominal_values(obj),
                             __ufloat_std__=std_devs(obj))
             else:
-                nomv=nominal_values(obj)
+                nomv = nominal_values(obj)
                 return dict(__udarray_tolist_avg__=nomv.tolist(),
                             __udarray_tolist_std__=std_devs(obj).tolist(),
                             dtype=str(nomv.dtype),
@@ -191,9 +198,11 @@ def json_dumper(obj, objects_encode=True):
                                 dtype=str(obj.dtype),
                                 shape=obj.shape)
         elif isinstance(obj, numpy.generic):
-            return numpy.asscalar(obj)
+            return obj.item()
         elif isinstance(obj, complex):
             return dict(__complex__=True, real=obj.real, imag=obj.imag)
+        elif isinstance(obj, bytes):
+            return obj.decode('utf-8')
         else:
             return obj.toJSON()
 
@@ -238,7 +247,7 @@ def json_loader(object_pairs, cls=dict):
         return uarray(numpy.array(dct['__udarray_tolist_avg__'], dtype=dct['dtype']).reshape(dct['shape']),
                       numpy.array(dct['__udarray_tolist_std__'], dtype=dct['dtype']).reshape(dct['shape']))
     elif '__ufloat__' in dct and '__ufloat_std__' in dct:
-        return ufloat(dct['__ufloat__'],dct['__ufloat_std__'])
+        return ufloat(dct['__ufloat__'], dct['__ufloat_std__'])
     elif '__ndarray__' in dct:
         import base64
         data = base64.b64decode(dct['__ndarray__'])
@@ -248,7 +257,7 @@ def json_loader(object_pairs, cls=dict):
     return dct
 
 
-def recursive_glob(pattern='*',rootdir='.'):
+def recursive_glob(pattern='*', rootdir='.'):
     """
     Search recursively for files matching a specified pattern within a rootdir
 
@@ -264,74 +273,6 @@ def recursive_glob(pattern='*',rootdir='.'):
     return matches
 
 
-def remote_uri(uri, filename, action):
-    """
-    :param uri: uri of the container of the file
-
-    :param filename: filename to act on
-
-    :param action: must be one of [`up`, `down`, `list`, `del`]
-    """
-    if not re.match('\w+://\w+.*', uri):
-        return uri
-
-    tmp = uri.split('://')
-    system = tmp[0]
-    location = '://'.join(tmp[1:])
-
-    if action not in ['down', 'up', 'list', 'del']:
-        raise (AttributeError('remote_uri action attribute must be one of [`up`, `down`, `list`, `del`]'))
-
-    if system == 's3':
-        import boto3
-        from boto3.s3.transfer import TransferConfig
-        s3bucket = location.split('/')[0]
-        s3connection = boto3.resource('s3')
-        s3filename = '/'.join(location.split('/')[1:])
-
-        if action == 'list':
-            printd('Listing %s' % (uri), topic='s3')
-            files=list(map(lambda x:x.key,s3connection.Bucket(s3bucket).objects.all()))
-            s3filename=s3filename.strip('/')
-            if s3filename:
-                files=filter(lambda x:x.startswith(s3filename),files)
-            return files
-
-        if action == 'del':
-            if filename is None:
-                filename = s3filename.split('/')[-1]
-            printd('Deleting %s' % uri, topic='s3')
-            s3connection.Object(s3bucket, s3filename).delete()
-
-        elif action == 'down':
-            if filename is None:
-                filename = s3filename.split('/')[-1]
-            printd('Downloading %s to %s' % (uri, filename), topic='s3')
-            obj = s3connection.Object(s3bucket, s3filename)
-            if not os.path.exists(os.path.abspath(os.path.split(filename)[0])):
-                os.makedirs(os.path.abspath(os.path.split(filename)[0]))
-            obj.download_file(filename,Config=TransferConfig(use_threads=False))
-
-        elif action == 'up':
-            printd('Uploading %s to %s' % (filename, uri), topic='s3')
-            from botocore.exceptions import ClientError
-            if s3filename.endswith('/'):
-                s3filename += filename.split('/')[-1]
-            try:
-                s3connection.meta.client.head_bucket(Bucket=s3bucket)
-            except ClientError as _excp:
-                # If a client error is thrown, then check that it was a 404 error.
-                # If it was a 404 error, then the bucket does not exist.
-                error_code = int(_excp.response['Error']['Code'])
-                if error_code == 404:
-                    s3connection.create_bucket(Bucket=s3bucket)
-                else:
-                    raise
-            bucket = s3connection.Bucket(s3bucket)
-            with open(filename, 'rb') as data:
-                bucket.put_object(Key=s3filename, Body=data)  # , Metadata=meta)
-
-
 def remove_parentheses(inv, replace_with=''):
     '''
     function used to remove/replace top-level matching parenthesis from a string
@@ -397,13 +338,13 @@ def closest_index(my_list, my_number=0):
     if pos == 0:
         return 0
     if pos == len(my_list):
-        return pos-1
+        return pos - 1
     before = pos - 1
     after = pos
     if my_list[after] - my_number < my_number - my_list[before]:
         return pos
     else:
-        return pos-1
+        return pos - 1
 
 
 def sanitize_version_number(version):
@@ -462,6 +403,7 @@ _coordinates = {}
 # }
 _extra_structures = {}
 
+
 def list_structures(imas_version):
     '''
     list names of structures in imas version
@@ -470,13 +412,14 @@ def list_structures(imas_version):
 
     :return: list with names of structures in imas version
     '''
-    json_filenames = glob.glob( imas_json_dir + os.sep + imas_versions.get(imas_version,imas_version) + os.sep + '*' + '.json')
-    json_filenames = filter(lambda x:os.path.basename(x)[0]!='_', json_filenames)
-    structures = sorted(list(map(lambda x: os.path.splitext(os.path.split(x)[1])[0],json_filenames)))
+    json_filenames = glob.glob(imas_json_dir + os.sep + imas_versions.get(imas_version, imas_version) + os.sep + '*' + '.json')
+    json_filenames = filter(lambda x: os.path.basename(x)[0] != '_', json_filenames)
+    structures = sorted(list(map(lambda x: os.path.splitext(os.path.split(x)[1])[0], json_filenames)))
     if not len(structures):
-        raise (ValueError("Unrecognized IMAS version `%s`. Possible options are:\n%s" % (imas_version, imas_versions.keys())))
+        raise ValueError("Unrecognized IMAS version `%s`. Possible options are:\n%s" % (imas_version, imas_versions.keys()))
     return structures
 
+
 def dict_structures(imas_version):
     '''
     maps structure names to json filenames
@@ -487,10 +430,11 @@ def dict_structures(imas_version):
     '''
     paths = glob.glob(imas_json_dir + os.sep + imas_versions.get(imas_version, imas_version) + os.sep + '*' + '.json')
     if not len(paths):
-        raise (ValueError("Unrecognized IMAS version `%s`. Possible options are:\n%s" % (imas_version, imas_versions.keys())))
+        raise ValueError("Unrecognized IMAS version `%s`. Possible options are:\n%s" % (imas_version, imas_versions.keys()))
     structures = dict(zip(list(map(lambda x: os.path.splitext(os.path.split(x)[1])[0], paths)), paths))
     return {structure: structures[structure] for structure in structures if not structure.startswith('_')}
 
+
 def load_structure(filename, imas_version):
     """
     load omas structure from given json filename or IDS name
@@ -505,7 +449,7 @@ def load_structure(filename, imas_version):
     from .omas_physics import cocos_signals
 
     filename0 = filename
-    id=(filename0, imas_version)
+    id = (filename0, imas_version)
     if id in _structures and id in _structures_dict:
         return _structures[id], _structures_dict[id]
 
@@ -516,15 +460,14 @@ def load_structure(filename, imas_version):
         with open(filename, 'r') as f:
             dump_string = f.read()
         # load flat definitions from json file
-        _structures[id] = json.loads(dump_string, object_pairs_hook=json_loader)
-        # _structures[id] = pickle.loads(dump_string)
+        _structures[id] = json.loads(dump_string)
 
         # add _extra_structures definitions
         structure_name = os.path.splitext(os.path.split(filename)[1])[0]
         if structure_name in _extra_structures:
             for item in _extra_structures[structure_name]:
                 if item not in _structures[id]:
-                    cs = _extra_structures[structure_name][item].pop('cocos_signal',None)
+                    cs = _extra_structures[structure_name][item].pop('cocos_signal', None)
                     _structures[id][item] = _extra_structures[structure_name][item]
                     if cs is not None:
                         cocos_signals[i2o(item)] = cs
@@ -551,14 +494,15 @@ def omas_coordinates(imas_version=omas_rcparams['default_imas_version']):
     '''
     # caching
     if imas_version not in _coordinates:
-        filename = imas_json_dir + os.sep + imas_versions.get(imas_version,imas_version) + os.sep + '_coordinates.json'
-        with open(filename,'r') as f:
-            _coordinates[imas_version] = json.loads(f.read(), object_pairs_hook=json_loader)
+        filename = imas_json_dir + os.sep + imas_versions.get(imas_version, imas_version) + os.sep + '_coordinates.json'
+        with open(filename, 'r') as f:
+            _coordinates[imas_version] = json.load(f)
     return _coordinates[imas_version]
 
 
 _p2l_cache = {}
 
+
 def p2l(key):
     """
     Converts the many different ways of addressing an ODS path to a list of keys (['bla',0,'bla'])
@@ -573,7 +517,7 @@ def p2l(key):
     if isinstance(key, tuple):
         return list(key)
 
-    if isinstance(key, (int,numpy.integer)):
+    if isinstance(key, (int, numpy.integer)):
         return [int(key)]
 
     if isinstance(key, basestring) and '.' not in key:
@@ -583,10 +527,10 @@ def p2l(key):
             return []
 
     if key is None:
-        raise(TypeError('OMAS key cannot be None'))
+        raise TypeError('OMAS key cannot be None')
 
     if isinstance(key, dict):
-        raise(TypeError('OMAS key cannot be of type dictionary'))
+        raise TypeError('OMAS key cannot be of type dictionary')
 
     key0 = ''.join(key)
     if key0 in _p2l_cache:
@@ -595,7 +539,7 @@ def p2l(key):
     if not isinstance(key, (list, tuple)):
         key = str(key).replace('[', '.').replace(']', '').split('.')
 
-    key = list(filter(None, key))
+    key = [k for k in key if k]
     for k, item in enumerate(key):
         try:
             key[k] = int(item)
@@ -619,7 +563,7 @@ def l2i(path):
     """
     ipath = path[0]
     for step in path[1:]:
-        if isinstance(step, int) or step==':':
+        if isinstance(step, int) or step == ':':
             ipath += "[%s]" % step
         else:
             ipath += '.%s' % step
@@ -635,8 +579,30 @@ def l2u(path):
 
     :return: universal ODS path format
     """
-    location = '.'.join(filter(None, map(str, path)))
-    return o2u(location)
+    return o2u(l2o(path))
+
+
+def l2ut(path):
+    """
+    Formats IMAS time lists (['bla',0,'time_slice',5,'quantity']) with universal ODS path ('bla.0.time_slice.:.quantity')
+
+    :param path: list of strings and integers
+
+    :return: ODS path format with time lists in universal format
+    """
+    lpath = p2l(path)
+    opath = l2o(lpath)
+    upath = o2u(path)
+    for k, key in enumerate(lpath):
+        if not isinstance(key, int):
+            continue
+        key = lpath[:k]
+        info = omas_info_node(l2u(key))
+        if 'coordinates' in info:
+            for infoc in info['coordinates']:
+                if infoc.endswith('.time'):
+                    lpath[k] = ':'
+    return l2o(lpath)
 
 
 def l2o(path):
@@ -649,8 +615,13 @@ def l2o(path):
     """
     return '.'.join(filter(None, map(str, path)))
 
+
 _o2u_pattern = re.compile('\.[0-9:]+')
 _o2u_pattern_no_split = re.compile('^[0-9:]+')
+_i2o_pattern = re.compile('\[([:0-9]+)\]')
+_o2i_pattern = re.compile('\.([:0-9]+)')
+
+
 def o2u(path):
     '''
     Converts an ODS path format ('bla.0.bla') into a universal path format ('bla.:.bla')
@@ -666,7 +637,6 @@ def o2u(path):
         return re.sub(_o2u_pattern_no_split, ':', path)
 
 
-_i2o_pattern=re.compile('\[([:0-9]+)\]')
 def i2o(path):
     """
     Formats a IMAS path ('bla[0].bla') format into an ODS path ('bla.0.bla')
@@ -678,7 +648,6 @@ def i2o(path):
     return re.sub(_i2o_pattern, r'.\1', path)
 
 
-_o2i_pattern=re.compile('\.([:0-9]+)')
 def o2i(path):
     """
     Formats a ODS path ('bla.0.bla') format into an IMAS path ('bla[0].bla')
@@ -692,8 +661,8 @@ def o2i(path):
 
 def u2o(upath, path):
     '''
-    Fills in `:` in a universal ODS path with integers from a ODS path.
-    e.g. uo2('bla.:.hello.:.bla','bla.1.hello.2.ddd')) becomes ('bla.1.hello.2.bla')
+    Replaces `:` and integers in `upath` with ':' and integers from in `path`
+    e.g. uo2('a.:.b.:.c.1.d.1.e','f.:.g.1.h.1.i.:.k')) becomes ('bla.1.hello.2.bla')
 
     :param upath: universal ODS path
 
@@ -703,10 +672,10 @@ def u2o(upath, path):
     '''
     if upath.startswith('1...'):
         return upath
-    ol = p2l(path)
     ul = p2l(upath)
+    ol = p2l(path)
     for k in range(min([len(ul), len(ol)])):
-        if ul[k] == ':' and isinstance(ol[k], int):
+        if (ul[k] == ':' or isinstance(ul[k], int)) and (ol[k] == ':' or isinstance(ol[k], int)):
             ul[k] = ol[k]
         elif ul[k] == ol[k]:
             continue
@@ -790,7 +759,7 @@ def omas_info(structures, imas_version=omas_rcparams['default_imas_version']):
         from omas import ODS
         _info_structures[imas_version] = ODS(imas_version=imas_version, consistency_check=False)
     ods = _info_structures[imas_version]
-    ods.consistency_check=False
+    ods.consistency_check = False
 
     for structure in structures:
         if structure not in ods:
@@ -806,7 +775,7 @@ def omas_info(structures, imas_version=omas_rcparams['default_imas_version']):
                         break
                 if parent:
                     continue
-                ods[item.replace(':','0')] = tmp[item]
+                ods[item.replace(':', '0')] = tmp[item]
 
     return copy.deepcopy(ods)
 
