@@ -21,6 +21,7 @@ from pprint import pprint
 from omas import *
 from omas.omas_setup import *
 
+
 class TestOmasCore(unittest.TestCase):
     """
     Test suite for omas_physics.py
@@ -52,9 +53,9 @@ class TestOmasCore(unittest.TestCase):
             ods['dataset_description.data_entry.user']
         except LookupError:
             ods['dataset_description'] = ODS()
-            ods['dataset_description.data_entry.user'] = os.environ['USER']
+            ods['dataset_description.data_entry.user'] = os.environ.get('USER', 'dummy_user')
         else:
-            raise (Exception('OMAS error handling dynamic_path_creation=False'))
+            raise Exception('OMAS error handling dynamic_path_creation=False')
         finally:
             ods.dynamic_path_creation = True
 
@@ -64,7 +65,7 @@ class TestOmasCore(unittest.TestCase):
         except ValueError:
             pass
         else:
-            raise (Exception('OMAS error querying leaf that has not been set'))
+            raise Exception('OMAS error querying leaf that has not been set')
 
         # info ODS is used for keeping track of IMAS metadata
         ods['dataset_description.data_entry.machine'] = 'ITER'
@@ -123,7 +124,7 @@ class TestOmasCore(unittest.TestCase):
         tmp = pickle.dumps(ods2)
         ods2 = pickle.loads(tmp)
         if ods2.consistency_check != ckbkp:
-            raise (Exception('consistency_check attribute changed'))
+            raise Exception('consistency_check attribute changed')
 
         # check flattening
         tmp = ods2.flat()
@@ -132,11 +133,41 @@ class TestOmasCore(unittest.TestCase):
         # check deepcopy
         ods3 = ods2.copy()
 
+        # check writing setting an xarray.DataArray
+        ods2['equilibrium.time_slice[2].profiles_1d.q'] = xarray.DataArray(uarray([0., 1., 2., 3.], [0, .1, .2, .3]), coords={'x': [1, 2, 3, 4]}, dims=['x'])
+
     def test_coordinates(self):
         ods = ods_sample()
         assert (len(ods.list_coordinates()) > 0)
         assert (len(ods['equilibrium'].list_coordinates()) > 0)
 
+    def test_dataset(self):
+        ods = ODS()
+
+        ods.sample_equilibrium(time_index=0)
+        ods.sample_equilibrium(time_index=1)
+
+        ods.sample_core_profiles(time_index=0)
+        ods.sample_core_profiles(time_index=1)
+
+        n = 1E10
+        sizes = {}
+        for homogeneous in [False, 'time', None, 'full']:
+            DS = ods.dataset(homogeneous=homogeneous)
+            print(homogeneous, len(DS.variables))
+            sizes[homogeneous] = len(DS.variables)
+            if homogeneous is not None:
+                assert n >= sizes[homogeneous], 'homogeneity setting does not match structure reduction expectation'
+                n = sizes[homogeneous]
+        assert sizes[None] == sizes['time'], 'sample equilibrium and core_profiles should be homogeneous'
+
+        ods.sample_pf_active()
+        try:
+            DS = ods.dataset(homogeneous='full')
+            raise AssertionError('sample pf_active data should not be able to collect across channels because their time arrays are not homogeneous')
+        except ValueError:
+            pass
+
     def test_time(self):
         # test generation of a sample ods
         ods = ODS()
@@ -149,9 +180,7 @@ class TestOmasCore(unittest.TestCase):
 
         # get time information from children
         extra_info = {}
-        assert numpy.allclose(ods.time('equilibrium', extra_info=extra_info), [100, 200, 300])
-        assert extra_info['location'] == 'equilibrium.time_slice.:.time'
-        assert extra_info['homogeneous_time'] is True
+        assert numpy.allclose(ods.time('equilibrium'), [100, 200, 300])
         assert ods['equilibrium'].homogeneous_time() is True
 
         # time arrays can be set using `set_time_array` function
@@ -168,34 +197,22 @@ class TestOmasCore(unittest.TestCase):
 
         # get time information from explicitly set time array
         extra_info = {}
-        assert numpy.allclose(ods.time('equilibrium', extra_info=extra_info), [101, 201, 302])
-        assert extra_info['homogeneous_time'] is False
-        assert ods['equilibrium'].homogeneous_time() is False
+        assert numpy.allclose(ods.time('equilibrium'), [101, 201, 302])
+        assert ods['equilibrium'].homogeneous_time() is True
 
         # get time value from a single item in array of structures
         extra_info = {}
-        assert ods['equilibrium.time_slice'][0].time(extra_info=extra_info) == 101
-        assert extra_info['homogeneous_time'] is None
-        tmp = ods['equilibrium']
-        assert tmp.homogeneous_time('time_slice.0') is True
-        assert ods['equilibrium'].homogeneous_time('time_slice.0', default=False) is False
+        assert ods['equilibrium.time_slice'][0].time() == 101
+        assert ods['equilibrium.time_slice'][0].homogeneous_time() is True
 
-        # get time array from array of structures
-        extra_info = {}
-        assert numpy.allclose(ods['equilibrium.time_slice'].time(extra_info=extra_info), [101, 201, 302])
-        assert extra_info['homogeneous_time'] is False
-        assert ods['equilibrium'].homogeneous_time() is False
+        # sample pf_active data has non-homogeneous times
+        ods.sample_pf_active()
+        assert ods['pf_active'].homogeneous_time() is False, 'sample pf_active data should have non-homogeneous time'
+        assert ods['pf_active.coil'][0]['current'].homogeneous_time() is True
 
-        # get time from parent
-        extra_info = {}
-        assert ods.time('equilibrium.time_slice.0.global_quantities.ip', extra_info=extra_info) == 101
-        assert extra_info['homogeneous_time'] is None
-        assert ods.homogeneous_time('equilibrium.time_slice.0.global_quantities.ip') is True
-        assert ods.homogeneous_time('equilibrium.time_slice.0.global_quantities.ip', default=False) is False
-
-        # slice at time
-        ods1 = ods['equilibrium'].slice_at_time(101)
-        numpy.allclose(ods.time('equilibrium'), [101])
+        ods.sample_dataset_description()
+        ods['dataset_description'].satisfy_imas_requirements()
+        assert ods['dataset_description.ids_properties.homogeneous_time'] is not None
 
     def test_address_structures(self):
         ods = ODS()
@@ -243,24 +260,16 @@ class TestOmasCore(unittest.TestCase):
             pass
 
     def test_satisfy_imas_requirements(self):
-        ods = ods_sample()
-
+        ods = ODS()
+        ods['equilibrium.time_slice.0.global_quantities.ip'] = 0.0
         # check if data structures satisfy IMAS requirements (this should Fail)
         try:
             ods.satisfy_imas_requirements()
-            raise (ValueError('It is expected that not all the sample structures have the .time array set'))
+            raise ValueError('It is expected that not all the sample structures have the .time array set')
         except ValueError as _excp:
             pass
 
-        # add .time information for all data structures
-        while True:
-            try:
-                ods.satisfy_imas_requirements()
-            except ValueError as _excp:
-                # print(str(_excp).split()[0])
-                ods[str(_excp).split()[0]] = [100]
-            else:
-                break
+        ods = ods_sample()
 
         # re-check if data structures satisfy IMAS requirements (this should pass)
         ods.satisfy_imas_requirements()
@@ -297,6 +306,7 @@ class TestOmasCore(unittest.TestCase):
         ods2['equilibrium.vacuum_toroidal_field.r0'] += 1
         assert ods['equilibrium.vacuum_toroidal_field.r0'] + 1 == ods2['equilibrium.vacuum_toroidal_field.r0']
 
+
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(TestOmasCore)
     unittest.TextTestRunner(verbosity=2).run(suite)
