@@ -375,8 +375,10 @@ class TestOmasPhysics(unittest.TestCase):
 
     @unittest.skipUnless(not failed_PINT, str(failed_PINT))
     def test_handle_units(self):
-        ods = ODS()
+        import pint
+        ureg = pint.UnitRegistry()
 
+        ods = ODS()
         ods['equilibrium.time_slice[0].constraints.diamagnetic_flux.time_measurement'] = 8.0 * ureg.milliseconds
         assert (ods['equilibrium.time_slice[0].constraints.diamagnetic_flux.time_measurement'] == 0.008)
 
@@ -397,7 +399,7 @@ class TestOmasPhysics(unittest.TestCase):
 
         try:
             tmp = search_ion(ods["core_profiles.profiles_1d.0.ion"])
-            raise (AssertError('multiple_matches_raise_error'))
+            raise AssertError('multiple_matches_raise_error')
         except IndexError:
             pass
 
@@ -409,7 +411,7 @@ class TestOmasPhysics(unittest.TestCase):
 
         try:
             tmp = search_ion(ods["core_profiles.profiles_1d.0.ion"], 'W')
-            raise (AssertError('no_matches_raise_error failed'))
+            raise AssertError('no_matches_raise_error failed')
         except IndexError:
             pass
 
@@ -427,7 +429,7 @@ class TestOmasPhysics(unittest.TestCase):
 
         try:
             search_in_array_structure(ods['core_transport.model'], {'identifier.name': 'omas_tgyro'})
-            raise (AssertionError('multiple_matches_raise_error failed'))
+            raise AssertionError('multiple_matches_raise_error failed')
         except IndexError:
             pass
 
@@ -437,6 +439,14 @@ class TestOmasPhysics(unittest.TestCase):
         tmp = search_in_array_structure(ods['core_transport.model'], {'identifier.name': 'omas_tgyro', 'identifier.description': 'bla bla'})
         assert tmp[0] == 2
 
+    def test_latest_cocos(self):
+        from omas.omas_utils import list_structures, omas_rcparams
+        from omas.omas_physics import generate_cocos_signals
+
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", message=".*defining its COCOS transform.*")
+            generate_cocos_signals(list_structures(imas_version=omas_rcparams['default_imas_version']), threshold=0, write=False, verbose=False)
+
 
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(TestOmasPhysics)
